@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -19,6 +19,7 @@
 #define __DRIVER_H_INCLUDED__
 
 #include <oskit/error.h>
+#include <oskit/dev/osenv.h>
 
 extern char eth_driver[];
 extern char packet[];
@@ -32,7 +33,7 @@ int eth_reset();
 int eth_transmit(char *, unsigned short, unsigned short, void *);
 int eth_poll();
 
-oskit_error_t net_init();
+oskit_error_t net_init(oskit_osenv_t *osenv);
 void net_shutdown();
 
 #endif /* __DRIVER_H_INCLUDED__ */
