@@ -36,5 +36,5 @@ Author: Martin Renters
 void convert_ipaddr(void *d, void *s);
 char *netsprintf(char *buf, char *fmt, ...);
 void netprintf(char *fmt, ...);
-int parse_cmdline(char *cmdline, unsigned int *ip, char **dir, char **name,
-		  char **rest);
+int parse_cmdline(char *cmdline, int cmdsize, unsigned int *ip, char **dir,
+		  char **name, char **rest);
