@@ -31,6 +31,15 @@ struct rpc_t {
 	} u;
 };
 
+struct rpc_buf {
+	int valid;
+	int sent;		   /* Is the request on the wire? */
+	unsigned long offset;
+	unsigned long len;
+	unsigned long id;
+	char buffer[1400];
+};
+
 #define SUNRPC		111
 
 #define RPC_SOCKET	620			/* Arbitrary */
@@ -60,7 +69,8 @@ void rpc_init(void);
 int rpc_lookup(int addr, int prog, int ver);
 int nfs_mount(int server, int port, char *path, char *fh);
 int nfs_lookup(int server, int port, char *fh, char *path, char *file_fh);
-int nfs_read(int server, int port, char *fh, int offset, int len, char *buffer);
+int nfs_read(int server, int port, char *fh, int offset, int len,
+	     char *buffer, int readahead);
 char *rpc_strerror(struct rpc_t *rpc);
 char *nfs_strerror(int err);
 
