@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -25,23 +25,51 @@
 #include <oskit/com/listener.h>
 #include <oskit/c/string.h>
 #include <oskit/c/stdlib.h>
-
 #include "pit_param.h"
 
 #define VERBOSITY	0
 
-#define BEGIN_CRITICAL_SECTION(flags)   \
-	flags = osenv_intr_enabled(); \
-	osenv_intr_disable();
+/*
+ * This file needs these osenv interfaces. The clock code is written on
+ * top of the osenv interfaces, and is intended to be standalone from the
+ * device library. It should be moved elsewhere, since it needs to use
+ * the external interfaces, not the internal ones.
+ */
+#include <oskit/dev/osenv.h>
+#include <oskit/dev/osenv_log.h>
+#include <oskit/dev/osenv_mem.h>
+#include <oskit/dev/osenv_intr.h>
+#include <oskit/dev/osenv_timer.h>
 
-#define END_CRITICAL_SECTION(flags) \
-	if (flags) osenv_intr_enable();
+/*
+ * Locally cached pointers to the COM objects needed.
+ */
+static oskit_osenv_log_t	*clock_osenv_log;
+
+/*
+ * For simplicity, lets map these older osenv_log calls to the new object
+ * invocations to avoid a ton of textual changes.
+ */
+#undef  osenv_assert
+#define osenv_assert(x) \
+	oskit_osenv_assert(clock_osenv_log, x)
+
+#define osenv_log(pri, fmt...) \
+	oskit_osenv_log_log(clock_osenv_log, (pri), ##fmt)
+
+
+#define BEGIN_CRITICAL_SECTION(intr,flags)   \
+	flags = oskit_osenv_intr_enabled(intr); \
+	oskit_osenv_intr_disable(intr);
+
+#define END_CRITICAL_SECTION(intr,flags) \
+	if (flags) oskit_osenv_intr_enable(intr);
 
 
 /*
  * What does a clock do, really?
  *
- * Two things: 
+ * Two things:
  *	a) it entertains a counter which can be queried for the
  * 	          current time.
  *
@@ -78,17 +106,24 @@ typedef struct clock_impl {
 	unsigned 		count;		/* reference count */
 	oskit_timespec_t 	time;		/* "current" time */
 
-	/* 
+        /*
+	 * Locally cached pointers to the COM objects needed.
+	 */
+	oskit_osenv_mem_t	*mem;
+	oskit_osenv_timer_t	*timer;
+	oskit_osenv_intr_t	*intr;
+
+	/*
 	 * Please see the distinction I made above.
 	 * Godmar feels it is important to optimize the case where a timer
 	 * expires at every clock tick
-	 * Thus, we keep two queues. Both of them are used by 
+	 * Thus, we keep two queues. Both of them are used by
 	 * the interrupt code, so changes to them must be protected.
 	 */
 
 	/* this is our "callout" queue for one-shot timers - a differential
-	 * list of events essentially. This list is ordered by 
-	 * timer_impl::left. Negative values indicate overdue events. 
+	 * list of events essentially. This list is ordered by
+	 * timer_impl::left. Negative values indicate overdue events.
 	 */
 	struct timer_impl	*callout;
 
@@ -107,8 +142,8 @@ clock_impl_t	*oskit_system_clock;
 
 /* forward declarations */
 static timer_impl_t * create_timer(clock_impl_t *clock);
-static void add_synchronous_timer(timer_impl_t **head, timer_impl_t *t);
-static void add_oneshot_timer(timer_impl_t **head, 
+static void add_synchronous_timer(oskit_osenv_intr_t *intr, timer_impl_t **head, timer_impl_t *t);
+static void add_oneshot_timer(oskit_osenv_intr_t *intr, timer_impl_t **head,
 	timer_impl_t *n, oskit_s32_t ticks);
 
 
@@ -138,7 +173,7 @@ static void fdev_handle_clock(clock_impl_t *clock);
 static void
 timer_intr()
 {
-	if (oskit_system_clock) 
+	if (oskit_system_clock)
 		fdev_handle_clock(oskit_system_clock);
 }
 
@@ -181,15 +216,18 @@ fdev_handle_clock(clock_impl_t *clock)
 		/* notify the object */
 		oskit_listener_notify(th->listener, (oskit_iunknown_t *)th);
 
-		/* 
+		/*
 		 * reload timer if necessary
 		 */
-		if (th->reload)
+		if (th->reload) {
 			if (th->reload == 1)
-				add_synchronous_timer(&clock->everytime, th);
+				add_synchronous_timer(clock->intr,
+						      &clock->everytime, th);
 			else
-				add_oneshot_timer(&clock->callout, th, 
-					th->reload);
+				add_oneshot_timer(clock->intr,
+						  &clock->callout, th,
+						  th->reload);
+		}
 	}
 }
 
@@ -214,25 +252,25 @@ print_queue(timer_impl_t **head)
  * add a timer to an unordered list of timers
  */
 static void
-add_synchronous_timer(timer_impl_t **head, timer_impl_t *t)
+add_synchronous_timer(oskit_osenv_intr_t *intr, timer_impl_t **head, timer_impl_t *t)
 {
 	unsigned int flags;
-	BEGIN_CRITICAL_SECTION(flags);
+	BEGIN_CRITICAL_SECTION(intr, flags);
 	t->clocklist = head;
 	t->next = *head;
 	*head = t;
 	t->left = t->reload;
-	END_CRITICAL_SECTION(flags);
+	END_CRITICAL_SECTION(intr, flags);
 }
 
 /*
  * remove a timer from an unordered list of timers
  */
 static void
-remove_synchronous_timer(timer_impl_t **head, timer_impl_t *t)
+remove_synchronous_timer(oskit_osenv_intr_t *intr, timer_impl_t **head, timer_impl_t *t)
 {
 	unsigned int flags;
-	BEGIN_CRITICAL_SECTION(flags);
+	BEGIN_CRITICAL_SECTION(intr, flags);
 	while (*head) {
 		if (*head == t) {
 			*head = t->next;
@@ -241,7 +279,7 @@ remove_synchronous_timer(timer_impl_t **head, timer_impl_t *t)
 		}
 		head = &(*head)->next;
 	}
-	END_CRITICAL_SECTION(flags);
+	END_CRITICAL_SECTION(intr, flags);
 }
 
 /*
@@ -249,12 +287,12 @@ remove_synchronous_timer(timer_impl_t **head, timer_impl_t *t)
  * this is programmed analogous to BSD's timeout function
  */
 static void
-add_oneshot_timer(timer_impl_t **head, timer_impl_t *n, oskit_s32_t ticks)
+add_oneshot_timer(oskit_osenv_intr_t *intr, timer_impl_t **head, timer_impl_t *n, oskit_s32_t ticks)
 {
 	timer_impl_t	*t, **p;
 	unsigned int flags;
 
-	BEGIN_CRITICAL_SECTION(flags);
+	BEGIN_CRITICAL_SECTION(intr, flags);
 	n->clocklist = head;
 
 	/* walk through list, finding spot, adjusting ticks parameter */
@@ -269,19 +307,19 @@ add_oneshot_timer(timer_impl_t **head, timer_impl_t *n, oskit_s32_t ticks)
 
 	n->next = t;
 	*p = n;
-	END_CRITICAL_SECTION(flags);
+	END_CRITICAL_SECTION(intr, flags);
 }
 
 /*
  * remove a one-shot timer, correcting the next queue entry
  */
 static void
-remove_oneshot_timer(timer_impl_t **head, timer_impl_t *f)
+remove_oneshot_timer(oskit_osenv_intr_t *intr, timer_impl_t **head, timer_impl_t *f)
 {
 	timer_impl_t    *t, **p;
 	unsigned int flags;
 
-	BEGIN_CRITICAL_SECTION(flags);
+	BEGIN_CRITICAL_SECTION(intr, flags);
 	for (p = head; (t = *p) != NULL; p = &t->next) {
 		if (t == f) {
 			if (t->next && t->left > 0)
@@ -292,7 +330,7 @@ remove_oneshot_timer(timer_impl_t **head, timer_impl_t *f)
 			break;
 		}
 	}
-	END_CRITICAL_SECTION(flags);
+	END_CRITICAL_SECTION(intr, flags);
 }
 
 /*
@@ -304,7 +342,7 @@ ticks_left_on_oneshot_timer(timer_impl_t *t, timer_impl_t *f)
 	unsigned int flags;
 	oskit_s32_t left = 0;
 
-	BEGIN_CRITICAL_SECTION(flags);
+	BEGIN_CRITICAL_SECTION(f->clock->intr, flags);
 
 	while (t) {
 		if (t->left > 0)
@@ -314,10 +352,10 @@ ticks_left_on_oneshot_timer(timer_impl_t *t, timer_impl_t *f)
 		t = t->next;
 	}
 
-	END_CRITICAL_SECTION(flags);
-	/* 
+	END_CRITICAL_SECTION(f->clock->intr,flags);
+	/*
 	 * if we didn't find t, lets return zero instead of returning
-	 * the accumulated sum of all time diffs 
+	 * the accumulated sum of all time diffs
 	 */
 	return t ? left : 0;
 }
@@ -328,11 +366,12 @@ ticks_left_on_oneshot_timer(timer_impl_t *t, timer_impl_t *f)
 static void
 remove_from_clocklist(timer_impl_t *ti)
 {
+	clock_impl_t *clock = ti->clock;
 	if (ti->clocklist == &ti->clock->everytime)
-		remove_synchronous_timer(&ti->clock->everytime, ti);
-	else 
+		remove_synchronous_timer(clock->intr, &clock->everytime, ti);
+	else
 	if (ti->clocklist == &ti->clock->callout)
-		remove_oneshot_timer(&ti->clock->callout, ti);
+		remove_oneshot_timer(clock->intr, &clock->callout, ti);
 }
 
 
@@ -367,7 +406,7 @@ timer_query(oskit_timer_t *io, const oskit_iid_t *iid, void **out_ihandle)
 
 
 /*
- * Clone a reference to a timer 
+ * Clone a reference to a timer
  */
 static OSKIT_COMDECL_U
 timer_addref(oskit_timer_t *io)
@@ -399,7 +438,7 @@ timer_release(oskit_timer_t *io)
 			remove_from_clocklist(ti);
 		if (ti->listener)
 			oskit_listener_release(ti->listener);
-		osenv_mem_free(ti, 0, sizeof(*ti));
+		oskit_osenv_mem_free(ti->clock->mem, ti, 0, sizeof(*ti));
 	}
 
 	return newcount;
@@ -422,7 +461,7 @@ timer_setlistener(oskit_timer_t *timer, struct oskit_listener *listener)
 
 /* Set the timer's expiration and period. */
 static OSKIT_COMDECL
-timer_settime(oskit_timer_t *timer, int flags, 
+timer_settime(oskit_timer_t *timer, int flags,
 	const struct oskit_itimerspec *value)
 {
 	timer_impl_t	*ti = (timer_impl_t *)timer;
@@ -466,21 +505,28 @@ timer_settime(oskit_timer_t *timer, int flags,
 	if (ti->clocklist)
 		remove_from_clocklist(ti);
 
-	/* 
+	/*
+	 * If left and reload are zero, do nothing. Might have been a
+	 * timer removal.
+	 */
+	if (left == 0 && reload == 0)
+		return 0;
+
+	/*
 	 * set reload in any event so that one-shot timers with a period
 	 * can be inserted in the periodic queue after they first expire
 	 */
 	ti->reload = reload;
 
 	/*
-	 * Step 3: Figure out just what kind of a timer we are 
+	 * Step 3: Figure out just what kind of a timer we are
 	 */
 	if (left == 1 && left == reload) {
 		/* this timer expires at every clock tick */
-		add_synchronous_timer(&clock->everytime, ti);
+		add_synchronous_timer(clock->intr, &clock->everytime, ti);
 	} else {
 		/* this timer starts out as a one-shot timer */
-		add_oneshot_timer(&clock->callout, ti, left);
+		add_oneshot_timer(clock->intr, &clock->callout, ti, left);
 	}
 
 	return 0;
@@ -498,13 +544,13 @@ timer_gettime(oskit_timer_t *timer, struct oskit_itimerspec *out_value)
 
 	/* set current period */
 	out_value->it_interval = ti->itimer.it_interval;
-	
+
 	/*
 	 * try to figure out how much time is left to next expiration
 	 */
 	if (ti->clocklist == &ti->clock->everytime)
 		left = 1;		/* one tick is left */
-	else 
+	else
 	if (ti->clocklist == &ti->clock->callout)
 		left = ticks_left_on_oneshot_timer(ti->clock->callout, ti);
 
@@ -579,8 +625,13 @@ clock_release(oskit_clock_t *io)
 
 	newcount = --po->count;
 	if (newcount == 0) {
-		osenv_timer_unregister(timer_intr, TIMER_FREQ);
-		osenv_mem_free(po, 0, sizeof(*po));
+		oskit_osenv_mem_t *mem = po->mem;
+		oskit_osenv_timer_unregister(po->timer,
+					     timer_intr, TIMER_FREQ);
+		oskit_osenv_timer_release(po->timer);
+		oskit_osenv_intr_release(po->intr);
+		oskit_osenv_mem_free(mem, po, 0, sizeof(*po));
+		oskit_osenv_mem_release(mem);
 	}
 
 	return newcount;
@@ -590,7 +641,7 @@ clock_release(oskit_clock_t *io)
 static OSKIT_COMDECL
 clock_getinfo(oskit_clock_t *fdev, oskit_devinfo_t *out_info)
 {
-	/* 
+	/*
 	 * this gives out info about the actual hardware device in use
 	 */
 	out_info->name = "clock";
@@ -604,13 +655,13 @@ clock_getinfo(oskit_clock_t *fdev, oskit_devinfo_t *out_info)
 static OSKIT_COMDECL
 clock_getdriver(oskit_clock_t *fdev, oskit_driver_t **out_driver)
 {
-	/* 
-	 * this a reference about the "more general" driver of 
+	/*
+	 * this a reference about the "more general" driver of
 	 * which we are an instance
 	 *
 	 * for now, just return an additional reference to us
 	 */
-	return clock_query(fdev, &oskit_clock_iid,(void **)out_driver);	
+	return clock_query(fdev, &oskit_clock_iid,(void **)out_driver);
 }
 
 /* Clock device interface operations */
@@ -647,7 +698,7 @@ clock_gettime(oskit_clock_t *dev, struct oskit_timespec *out_time)
 static OSKIT_COMDECL_V
 clock_sleep(oskit_clock_t *dev, const struct oskit_timespec *time)
 {
-	/* 
+	/*
 	 * this would be implemented by posting an "internal"
 	 * timeout and then sleeping
 	 */
@@ -661,7 +712,8 @@ clock_sleep(oskit_clock_t *dev, const struct oskit_timespec *time)
 static OSKIT_COMDECL_V
 clock_spin(oskit_clock_t *dev, const struct oskit_timespec *time)
 {
-	osenv_timer_spin(TIMESPEC2NANOSEC(time));
+	clock_impl_t *po = (clock_impl_t *)dev;
+	oskit_osenv_timer_spin(po->timer, TIMESPEC2NANOSEC(time));
 }
 
 /*
@@ -704,7 +756,7 @@ clock_settime(oskit_clock_t *dev, const struct oskit_timespec *new_time)
  * so that the clock's value does not jump or go backward.
  * Clocks that aren't adjustable will return OSKIT_E_NOTIMPL.
  */
-static OSKIT_COMDECL    
+static OSKIT_COMDECL
 clock_adjtime(oskit_clock_t *dev, const struct oskit_timespec *new_time)
 {
 	/* there's no way I'm going to implement that... */
@@ -724,7 +776,7 @@ clock_adjtime(oskit_clock_t *dev, const struct oskit_timespec *new_time)
 static OSKIT_COMDECL
 clock_setperiod(oskit_clock_t *dev, const struct oskit_timespec *period)
 {
-	return OSKIT_E_NOTIMPL;	
+	return OSKIT_E_NOTIMPL;
 }
 
 /*
@@ -749,9 +801,9 @@ clock_open(oskit_clock_t *dev, oskit_clock_t **out_opendev)
 /*
  * vtables
  */
-struct oskit_clock_ops oskit_clock_ops = {
+static struct oskit_clock_ops oskit_clock_ops = {
 	clock_query,
-	clock_addref, 
+	clock_addref,
 	clock_release,
 	clock_getinfo,
 	clock_getdriver,
@@ -766,7 +818,7 @@ struct oskit_clock_ops oskit_clock_ops = {
 	clock_open
 };
 
-struct oskit_timer_ops oskit_timer_ops = {
+static struct oskit_timer_ops oskit_timer_ops = {
 	timer_query,
 	timer_addref,
 	timer_release,
@@ -780,7 +832,8 @@ static timer_impl_t *
 create_timer(clock_impl_t *clock)
 {
 	timer_impl_t *t;
-	t = (timer_impl_t *)osenv_mem_alloc(sizeof *t, 0, 0);
+	t = (timer_impl_t *)oskit_osenv_mem_alloc(clock->mem,
+						  sizeof *t, 0, 0);
 	memset(t, 0, sizeof *t);
 	t->iot.ops = &oskit_timer_ops;
 	t->count = 1;
@@ -795,6 +848,10 @@ oskit_clock_t *
 oskit_clock_init()
 {
 	clock_impl_t	*c;
+	oskit_osenv_t	*osenv;
+	oskit_osenv_mem_t   *mem;
+	oskit_osenv_timer_t *timer;
+	oskit_osenv_intr_t  *intr;
 
 	/* there can be only one */
 	if (oskit_system_clock)	{
@@ -802,19 +859,41 @@ oskit_clock_init()
 		return &oskit_system_clock->ioc;
 	}
 
+	/*
+	 * Find the osenv, and then the various osenv interfaces this
+	 * component needs.
+	 */
+	oskit_lookup_first(&oskit_osenv_iid, (void *) &osenv);
+
+	oskit_osenv_lookup(osenv,
+			   &oskit_osenv_mem_iid, (void *) &mem);
+	oskit_osenv_lookup(osenv,
+			   &oskit_osenv_intr_iid, (void *) &intr);
+	oskit_osenv_lookup(osenv,
+			   &oskit_osenv_timer_iid, (void *)&timer);
+	oskit_osenv_lookup(osenv,
+			   &oskit_osenv_log_iid, (void *) &clock_osenv_log);
+
+	oskit_osenv_release(osenv);
+
 	/*
 	 * initialize the COM object
 	 */
-	c = (clock_impl_t *)osenv_mem_alloc(sizeof(*c), 0, 0);
+	c = (clock_impl_t *)oskit_osenv_mem_alloc(mem,
+						  sizeof(*c), 0, 0);
         if (c == NULL)
                 return NULL;
 
 	memset(c, 0, sizeof *c);
         c->ioc.ops = &oskit_clock_ops;
         c->count = 1;
+        c->mem   = mem;
+        c->timer = timer;
+        c->intr  = intr;
 
-	osenv_timer_init();	/* safe to call multiple times */
+	/* safe to call multiple times */
+	oskit_osenv_timer_init(timer);
 	oskit_system_clock = c;
-	osenv_timer_register(timer_intr, TIMER_FREQ);
+	oskit_osenv_timer_register(timer, timer_intr, TIMER_FREQ);
 	return &c->ioc;
 }
