@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -19,17 +19,22 @@
  */
 
 #include <oskit/dev/dev.h>
+#include <oskit/dev/osenv.h>
+#include <oskit/dev/driver.h>
+#include <oskit/dev/device.h>
 #include <oskit/debug.h>
 
 void
-oskit_dev_init(void)
+oskit_dev_init(oskit_osenv_t *osenv)
 {
-	static fdev_inited = 0;
+	static int fdev_inited = 0;
 
 	/* Only initialize once! */
 	if (fdev_inited)
 		return;
-	else  
+	else
 		fdev_inited = 1;
 
+	osenv_device_registration_init();
+	osenv_driver_registration_init();
 }
