@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -42,10 +42,6 @@
  * We could use the bmodfs but we use the netbsd filesystem to test more stuff.
  */
 
-/* Set these accordingly.  Also, make sure main() inits the appropriate drivers. */
-#define DISKNAME "wd1"
-#define PARTITION "b"
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -64,6 +60,7 @@
 #include <fcntl.h>
 #include <oskit/threads/pthread.h>
 #include <oskit/startup.h>
+#include <oskit/clientos.h>
 #include <assert.h>
 #include <sys/stat.h>
 #include <oskit/queue.h>
@@ -72,9 +69,11 @@
 int		proxy_socket;
 int		death_socket;
 int		create_listener_socket(int port);
-void		deathwatch(void *foo);
+void		*deathwatch(void *foo);
 pthread_mutex_t gethostname_mutex;
 
+int           threads_stack_back_trace(int, int);
+
 #include <oskit/dev/dev.h>
 #include <oskit/dev/linux.h>
 #ifdef  OSKIT_UNIX
@@ -92,12 +91,12 @@ typedef struct _connection {
 	int		died;		/* One side or the other died. */
 } connection_t;
 
-void	connection_manager(void *arg);
+void	*connection_manager(void *arg);
 
 /*
  * Act as simple HTTP proxy.
  */
-void
+int
 main()
 {
 	int		length, sock;
@@ -106,28 +105,25 @@ main()
 	extern int	threads_debug;
 
 	/*
-	 * Initialize the pthreads module.
+	 * Init the OS.
 	 */
-	pthread_init(1);
-	threads_debug = 0;
-
-#ifndef OSKIT_UNIX
+	oskit_clientos_init_pthreads();
 	start_clock();
-	osenv_process_lock();
-	oskit_linux_init_ethernet_tulip();
-	oskit_linux_init_ethernet_eepro100();
-	oskit_linux_init_ide();
-	oskit_dev_probe();
-	osenv_process_unlock();
-	start_fs_pthreads(DISKNAME, PARTITION);
+	start_pthreads();
+	threads_debug = 0;
+	
+#ifndef OSKIT_UNIX
+	start_world_pthreads();
+#else
+#ifdef  REALSTUFF
+	start_fs_pthreads("/dev/oskitfs", 0);
 	start_network_pthreads();
 #else
-	/*
-	 * Unix mode.
-	 */
-	start_fs_native_pthreads("/");
+	start_fs_native_pthreads("/tmp");
 	start_network_native_pthreads();
 #endif
+#endif
+
 	pthread_mutex_init(&gethostname_mutex, 0);
 	proxy_socket = create_listener_socket(6969);
 	death_socket = create_listener_socket(6666);
@@ -155,6 +151,7 @@ main()
 		pthread_create(&tid, 0, connection_manager, (void *) sock);
 		pthread_detach(tid);
 	}
+	return 0;
 }
 
 /*
@@ -244,7 +241,9 @@ connect_to_host(char *host, char *port, int *server_sock)
 	if (setsockopt(sock,
 		       SOL_SOCKET, SO_RCVTIMEO, &timeo, sizeof(timeo)) < 0) {
 		perror("setsockopt SO_RCVTIMEO");
+#if 0
 		panic("SETSOCKOPT");
+#endif
 	}
 
 	/*
@@ -301,7 +300,9 @@ setup_connection(int client_sock, connection_t **return_connp)
 	if (setsockopt(client_sock,
 		       SOL_SOCKET, SO_RCVTIMEO, &timeo, sizeof(timeo)) < 0) {
 		perror("setsockopt SO_RCVTIMEO");
+#if 0
 		return 1;
+#endif
 	}
 
 	/*
@@ -314,7 +315,7 @@ setup_connection(int client_sock, connection_t **return_connp)
 				printf("Closing down Client side\n");
 				return 1;
 			}
-			if (errno) {
+			if (errno && errno != EINTR) {
 				perror("First read on client side");
 				return 1;
 			}
@@ -334,7 +335,7 @@ setup_connection(int client_sock, connection_t **return_connp)
 		printf("Bad URL(1): %s\n", buf);
 		return 1;
 	}
-	printf("%d: %s %s\n", pthread_self(), method, url);
+	printf("%d: %s %s\n", (int) pthread_self(), method, url);
 	rest = &url[strlen(url)];
 	rest++;
 	
@@ -354,12 +355,13 @@ setup_connection(int client_sock, connection_t **return_connp)
 		host = url+7;
 	}
 	else if (strncasecmp(method, "CONNECT", 7) == 0) {
-		printf("%d: %s\n", pthread_self(), rest);
+		printf("%d: %s\n", (int) pthread_self(), rest);
 		host      = url;
 		tunneling = 1;
 	}
 	else {
-		printf("%d: Bad METHOD: %s %s\n", pthread_self(), method, url);
+		printf("%d: Bad METHOD: %s %s\n",
+		       (int) pthread_self(), method, url);
 		return 1;
 	}
 
@@ -451,16 +453,16 @@ setup_connection(int client_sock, connection_t **return_connp)
 	return 0;
 }
 
-void
+void *
 connection_manager(void *arg)
 {
-	void		client_side(void *connp);
-	void		server_side(void *connp);
+	void		*client_side(void *connp);
+	void		*server_side(void *connp);
 	pthread_t	client_tid, server_tid;
 	int		status, client_sock = (int) arg;
 	connection_t	*connp;
 
-	printf("Connection accepted. tid=%d\n", pthread_self());
+	printf("Connection accepted. tid=%d\n", (int) pthread_self());
 
 	if (setup_connection(client_sock, &connp)) {
 		close(client_sock);
@@ -495,7 +497,7 @@ connection_manager(void *arg)
 
 	printf("Manager: tid=%d "
 	       "Client(%d) and server(%d) side threads have exited\n",
-	       pthread_self(), client_tid, server_tid);
+	       (int) pthread_self(), (int) client_tid, (int) server_tid);
 
 	shutdown(connp->client_sock, 2);
 	shutdown(connp->server_sock, 2);
@@ -504,32 +506,50 @@ connection_manager(void *arg)
 
 	free(connp);
 
-	pthread_exit(0);
+	return 0;
+}
+
+/*
+ * Since the POSIX library does not implement cancelation points at
+ * all the necessary spots, need to do this ourselves.
+ */
+void
+cleanup_handler(void *arg)
+{
+	connection_t	*connp = (connection_t *) arg;
+
+	pthread_mutex_lock(&connp->mutex);
+	connp->died++;
+	pthread_mutex_unlock(&connp->mutex);
+	pthread_cond_signal(&connp->condvar);
 }
 
 /*
  * Read from the client side of the connection and send to the server
  * side.
  */
-void
+void *
 client_side(void *arg)
 {
 	connection_t	*connp = (connection_t *) arg;
 	char		buf[4096];
 	int		c;
 
-	printf("client_side starting: tid=%d\n", pthread_self());
+	printf("client_side starting: tid=%d\n", (int) pthread_self());
 
 	pthread_mutex_lock(&connp->mutex);
 	pthread_mutex_unlock(&connp->mutex);
+
+	pthread_cleanup_push(cleanup_handler, connp);
 	
 	while (1) {
+		pthread_testcancel();
 		if ((c = recv(connp->client_sock, buf, 4096, 0)) <= 0) {
 			if (c == 0) {
 				printf("Read: Closing Client side\n");
 				goto done;
 			}
-			else if (errno == EAGAIN) {
+			else if (errno == EAGAIN || errno == EINTR) {
 				/*
 				 * On client side, a read timeout means no
 				 * data available. Thats okay.
@@ -558,30 +578,30 @@ client_side(void *arg)
 		}
 	}
    done:
-	pthread_mutex_lock(&connp->mutex);
-	connp->died++;
-	pthread_mutex_unlock(&connp->mutex);
-	pthread_cond_signal(&connp->condvar);
-	pthread_exit(0);
+	pthread_cleanup_pop(1);
+	return 0;
 }
 
 /*
  * Read from the server side of the connection and send to the client
  * side.
  */
-void
+void *
 server_side(void *arg)
 {
 	connection_t	*connp = (connection_t *) arg;
 	char		buf[4096];
 	int		c;
 	
-	printf("server_side starting: tid=%d\n", pthread_self());
+	printf("server_side starting: tid=%d\n", (int) pthread_self());
 
 	pthread_mutex_lock(&connp->mutex);
 	pthread_mutex_unlock(&connp->mutex);
 
+	pthread_cleanup_push(cleanup_handler, connp);
+	
 	while (1) {
+		pthread_testcancel();
 		if ((c = recv(connp->server_sock, buf, 4096, 0)) <= 0) {
 			if (c == 0) {
 				printf("Read: Closing Server side\n");
@@ -593,7 +613,7 @@ server_side(void *arg)
 				 * data available. Thats okay.
 				 */
 				printf("Read: EAGAIN on server side: tid=%d\n",
-				       pthread_self());
+				       (int) pthread_self());
 				goto done;
 			}
 			else {
@@ -601,6 +621,7 @@ server_side(void *arg)
 				goto done;
 			}
 		}
+		/*printf("Server %d\n%s\n", c, buf);*/
 		if ((c = send(connp->client_sock, buf, c, 0)) != c) {
 			if (c == 0) {
 				printf("Send: Closing Server side\n");
@@ -617,14 +638,11 @@ server_side(void *arg)
 		}
 	}
    done:
-	pthread_mutex_lock(&connp->mutex);
-	connp->died++;
-	pthread_mutex_unlock(&connp->mutex);
-	pthread_cond_signal(&connp->condvar);
-	pthread_exit(0);
+	pthread_cleanup_pop(1);
+	return 0;
 }
 
-void
+void *
 deathwatch(void *arg)
 {
 	char		buf[4096];
@@ -659,10 +677,12 @@ deathwatch(void *arg)
 
 			if (strncmp(buf, "reboot", 6) == 0)
 				exit(1);
+#if !defined(OSKIT_UNIX) && defined(OSKIT_X86)
 			if (strncmp(buf, "gdb", 3) == 0) {
 				asm("int $3");
 				base_gdt_load();
 			}
+#endif
 			if (strncmp(buf, "bta", 3) == 0) {
 				int i;
 				for (i = 0; i < 100; i++) {
@@ -671,12 +691,12 @@ deathwatch(void *arg)
 			}
 			else if (strncmp(buf, "bt", 2) == 0) {
 				buf[4] = 0;
-				threads_stack_back_trace(atoi(buf));
+				threads_stack_back_trace(atoi(buf), 24);
 			}
 		}
            done:
 	}
-	exit(0);
+	return 0;
 }
 
 /*
