@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -15,10 +15,12 @@
  * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
  */
 
-#include <oskit/c/fd.h>
-
 /*
- * Initialize the C library.
+ * Initialize the C library. This call is required in multithreaded
+ * kernels, but is actually called out of the threads library.
+ *
+ * In singlethreaded kernels, this stuff is done lazily, so no need to
+ * call it. 
  */
 void
 oskit_init_libc(void)
@@ -28,8 +30,8 @@ oskit_init_libc(void)
 	if (didit)
 		return;
 	
-	fd_init();
+	oskit_posixlib_init();
 	mem_lock_init();
-	signals_init();
+
 	didit = 1;
 }
