@@ -2,6 +2,9 @@
  * Use the oskit bits which are different from the BSD bits.
  */
 
+#ifndef _SYS_FCNTL_H_
+#define _SYS_FCNTL_H_
+
 #include <sys/types.h>
 #include <oskit/c/fcntl.h>
 
@@ -38,3 +41,6 @@ struct flock {
 #define	LOCK_NB		0x04		/* don't block when locking */
 #define	LOCK_UN		0x08		/* unlock file */
 #endif
+
+#endif /* _SYS_FCNTL_H_ */
+
