@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1994-1995, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1994-1995, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -19,35 +19,13 @@
 
 #include "malloc.h"
 
-/* XX could be made smarter, so it doesn't always copy to a new block.  */
 void *realloc(void *buf, oskit_size_t new_size)
 {
-	oskit_size_t *op;
-	oskit_size_t old_size;
-	oskit_size_t *np;
+	void	*chunk;
 
 	if (buf == 0)
 		return malloc(new_size);
 
-	op = (oskit_size_t*)buf;
-	old_size = *--op;
-
-	new_size += sizeof(oskit_size_t);
-	mem_lock();
-	while (!(np = lmm_alloc(&malloc_lmm, new_size, 0)))
-	{
-		mem_unlock();
-		if (!morecore(new_size))
-			return 0;
-		mem_lock();
-	}
-
-	memcpy(np, op, old_size < new_size ? old_size : new_size);
-
-	lmm_free(&malloc_lmm, op, old_size);
-	mem_unlock();
-	
-	*np++ = new_size;
-	return np;
+	return oskit_mem_realloc(libc_memory_object, buf,
+				 0, new_size, OSKIT_MEM_AUTO_SIZE);
 }
-
