@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -29,6 +29,7 @@
 
 void *smemalign(size_t alignment, size_t size)
 {
-	return smemalignf(alignment, size, 0);
-}
+	CHECK_MEMORY_OBJECT();
 
+	return oskit_mem_alloc_aligned(libc_memory_object, size, 0, alignment);
+}
