@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -23,6 +23,15 @@
  * since it is based on the deprecated bsdnet_ interfaces.
  */
 
+#if FLASK
+#include <oskit/comsid.h>
+#include "netflask.h"
+#endif
+
+#ifndef offsetof
+#define offsetof(type, member) ((size_t)(&((type *)0)->member))
+#endif
+
 #include <oskit/dev/dev.h>
 #include <oskit/net/freebsd.h>
 #include <oskit/net/socket.h>
@@ -32,13 +41,17 @@
 
 #include <oskit/c/string.h>
 #include <sys/param.h>
-#include <sys/mbuf.h>
-#include <sys/proc.h>
 #include <sys/systm.h>
-#include <sys/select.h>
+#include <sys/errno.h>
+#include <sys/kernel.h>
+#include <sys/malloc.h>
+#include <sys/proc.h>
+#include <sys/mbuf.h>
 #include <sys/socket.h>
-#include <sys/protosw.h>
 #include <sys/socketvar.h>
+#include <sys/select.h>
+#include <sys/protosw.h>
+#include <sys/uio.h>
 #include <oskit/dev/freebsd.h>	/* declares xlate_errno */
 
 #include "glue.h"
@@ -104,6 +117,16 @@ socket_query(oskit_socket_t *f, const struct oskit_guid *iid, void **out_ihandle
 	return query(b, iid, out_ihandle);
 }
 
+#if FLASK
+static OSKIT_COMDECL
+secsocket_query(oskit_socket_secure_t *f, const struct oskit_guid *iid, void **out_ihandle)
+{
+	oskit_sockimpl_t *b = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+	return query(b, iid, out_ihandle);
+}
+#endif
+
 static OSKIT_COMDECL
 opensocket_query(oskit_stream_t *f, const struct oskit_guid *iid,
 				 void **out_ihandle)
@@ -148,6 +171,16 @@ socket_addref(oskit_socket_t *f)
 	return addref(b);
 }
 
+#if FLASK
+static OSKIT_COMDECL_U
+secsocket_addref(oskit_socket_secure_t *f)
+{
+	oskit_sockimpl_t *b = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+	return addref(b);
+}
+#endif
+
 static OSKIT_COMDECL_U
 opensocket_addref(oskit_stream_t *f)
 {
@@ -192,9 +225,9 @@ release(oskit_sockimpl_t *si)
 		 */
 		si->so->so_rcv.sb_sel.si_sel = 0;
 		si->so->so_snd.sb_sel.si_sel = 0;
-		OSKIT_FREEBSD_CREATE_CURPROC(p)
+		OSKIT_FREEBSD_CREATE_CURPROC(p);
 		soclose(si->so);
-		OSKIT_FREEBSD_DESTROY_CURPROC(p)
+		OSKIT_FREEBSD_DESTROY_CURPROC(p);
 		oskit_destroy_listener_mgr(si->readers);
 		oskit_destroy_listener_mgr(si->writers);
                 osenv_mem_free(si, 0, sizeof(*si));
@@ -209,6 +242,16 @@ socket_release(oskit_socket_t *f)
         return release(b);
 }
 
+#if FLASK
+static OSKIT_COMDECL_U
+secsocket_release(oskit_socket_secure_t *f)
+{
+	oskit_sockimpl_t *b = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+        return release(b);
+}
+#endif
+
 static OSKIT_COMDECL_U
 opensocket_release(oskit_stream_t *f)
 {
@@ -281,6 +324,16 @@ static OSKIT_COMDECL
 socket_bind(oskit_socket_t *s, const struct oskit_sockaddr *name,
 	oskit_size_t namelen)
 {
+#if FLASK
+        oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, SETLOCAL, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return xlaterc(bsdnet_bind(((oskit_sockimpl_t *)s)->so,
 				   (struct sockaddr *)name, namelen));
 }
@@ -290,6 +343,15 @@ socket_connect(oskit_socket_t *s, const struct oskit_sockaddr *name,
 	oskit_size_t namelen)
 {
 	oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+#if FLASK
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, SETREMOTE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return xlaterc(bsdnet_connect(b->so,
 				      (struct sockaddr *)name, namelen));
 }
@@ -297,12 +359,24 @@ socket_connect(oskit_socket_t *s, const struct oskit_sockaddr *name,
 static OSKIT_COMDECL
 socket_listen(oskit_socket_t *s, oskit_u32_t n)
 {
+	oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
 	struct proc p;
 	int    rc;
+#if FLASK
+        security_id_t csid;
+	
+	CSID(&csid);
+	STRSOCHECK(csid, b->so, LISTEN, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
 
-	OSKIT_FREEBSD_CREATE_CURPROC(p)
-	rc = solisten(((oskit_sockimpl_t *)s)->so, n);
-	OSKIT_FREEBSD_DESTROY_CURPROC(p)
+	b->so->so_newconn_sid = b->so->so_sid;
+	b->so->so_options &= ~SO_SIDREFLECT;
+#endif
+
+	OSKIT_FREEBSD_CREATE_CURPROC(p);
+	rc = solisten(b->so, n, &p);
+	OSKIT_FREEBSD_DESTROY_CURPROC(p);
 	return xlaterc(rc);
 }
 
@@ -310,6 +384,16 @@ static OSKIT_COMDECL
 socket_getsockname(oskit_socket_t *s, struct oskit_sockaddr *asa,
 	oskit_size_t *alen)
 {
+#if FLASK
+        oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, GETLOCAL, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return xlaterc(bsdnet_getsockname(((oskit_sockimpl_t *)s)->so,
 					  (struct sockaddr *)asa, alen));
 }
@@ -318,6 +402,19 @@ static OSKIT_COMDECL
 socket_setsockopt(oskit_socket_t *s, oskit_u32_t level, oskit_u32_t name,
 		const void *val, oskit_size_t valsize)
 {
+#if FLASK
+        oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	if (level == OSKIT_SOL_SOCKET)
+		COMMONSOCHECK(csid, b->so, SETOPT, &rc);
+	else
+		COMMONSOCHECK(csid, b->so, SETPROTOOPT, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return xlaterc(bsdnet_setsockopt(((oskit_sockimpl_t *)s)->so,
 					 level, name, val, valsize));
 }
@@ -327,7 +424,17 @@ socket_sendto(oskit_socket_t *s, const void *msg, oskit_size_t len,
 		oskit_u32_t flags, const struct oskit_sockaddr *to,
 		oskit_size_t tolen, oskit_size_t *retval)
 {
-	return xlaterc(bsdnet_sendto(((oskit_sockimpl_t *)s)->so,
+	oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+#if FLASK
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, SEND, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
+	return xlaterc(bsdnet_sendto(b->so,
 				     msg, len, flags,
 				     (struct sockaddr *)to, tolen, retval));
 }
@@ -338,6 +445,16 @@ socket_recvfrom(oskit_socket_t *s, void *buf,
 		struct oskit_sockaddr *from, oskit_size_t *fromlen,
 		oskit_size_t *retval)
 {
+#if FLASK
+	oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, RECEIVE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return xlaterc(bsdnet_recvfrom(((oskit_sockimpl_t *)s)->so,
 				       buf, len, flags,
 				       (struct sockaddr *)from, fromlen,
@@ -348,6 +465,16 @@ static OSKIT_COMDECL
 socket_getsockopt(oskit_socket_t *s, oskit_u32_t level, oskit_u32_t name,
 		void *val, oskit_size_t *valsize)
 {
+#if FLASK
+        oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, GETOPT, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return OSKIT_E_NOTIMPL;
 }
 
@@ -355,6 +482,16 @@ static OSKIT_COMDECL
 socket_sendmsg(oskit_socket_t *s, const struct oskit_msghdr *msg,
 		oskit_u32_t flags, oskit_size_t *retval)
 {
+#if FLASK
+	oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, SEND, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return OSKIT_E_NOTIMPL;
 }
 
@@ -362,6 +499,16 @@ static OSKIT_COMDECL
 socket_recvmsg(oskit_socket_t *s, struct oskit_msghdr *msg,
 		oskit_u32_t flags, oskit_size_t *retval)
 {
+#if FLASK
+	oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, RECEIVE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return OSKIT_E_NOTIMPL;
 }
 
@@ -370,6 +517,15 @@ socket_getpeername(oskit_socket_t *f,
 	struct oskit_sockaddr *asa, oskit_size_t *alen)
 {
 	oskit_sockimpl_t *s = (oskit_sockimpl_t *)f;
+#if FLASK
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, s->so, GETREMOTE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return xlaterc(bsdnet_getpeername(s->so,
 					  (struct sockaddr *)asa, alen));
 }
@@ -379,13 +535,193 @@ socket_shutdown(oskit_socket_t *f, oskit_u32_t how)
 {
 	struct proc p;
 	int    rc;
+#if FLASK
+	oskit_sockimpl_t *b = (oskit_sockimpl_t *)f;
+        security_id_t csid;
+	
+	CSID(&csid);
+	switch (how)
+	{
+	  case 0:
+	    COMMONSOCHECK(csid, b->so, DISABLE_RECEIVE, &rc);
+	    break;
+	  case 1:
+	    COMMONSOCHECK(csid, b->so, DISABLE_SEND, &rc);
+	    break;
+	  case 2:
+	    COMMONSOCHECK2(csid, b->so, DISABLE_RECEIVE, DISABLE_SEND, &rc);
+	    break;
+	  default:
+	    return OSKIT_E_INVALIDARG;
+	}
 
-	OSKIT_FREEBSD_CREATE_CURPROC(p)
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
+
+	OSKIT_FREEBSD_CREATE_CURPROC(p);
 	rc = soshutdown(((oskit_sockimpl_t *)f)->so, how);
-	OSKIT_FREEBSD_DESTROY_CURPROC(p)
+	OSKIT_FREEBSD_DESTROY_CURPROC(p);
 	return xlaterc(rc);
 }
 
+
+#if FLASK
+/*** Operations specific to oskit_socket_secure_t ***/
+static OSKIT_COMDECL secsocket_accept(oskit_socket_secure_t *f, 
+				      struct oskit_sockaddr *name,
+				      oskit_size_t *anamelen,
+				      security_id_t *out_peer_sid,
+				      struct oskit_socket **newopenso)
+{
+	oskit_sockimpl_t *s = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+	oskit_error_t rc;
+
+
+	rc = socket_accept(&s->ioi, name, anamelen, newopenso);
+	if (rc)
+		return rc;
+
+	*out_peer_sid = s->so->so_peer_sid;
+	return 0;
+}
+
+
+static OSKIT_COMDECL
+secsocket_connect(oskit_socket_secure_t *f, const struct oskit_sockaddr *name,
+		  oskit_size_t namelen, security_id_t peer_sid)
+{
+	oskit_sockimpl_t *s = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, s->so, SETREMOTE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+
+	return xlaterc(bsdnet_connect_secure(s->so,
+				     (struct sockaddr *)name, namelen,
+				     peer_sid));
+}
+
+
+static OSKIT_COMDECL
+secsocket_listen(oskit_socket_secure_t *f, oskit_u32_t n,
+		 security_id_t newconn_sid, oskit_u32_t flags)
+{
+	oskit_sockimpl_t *s = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+        security_id_t csid;
+	int rc;
+	
+	if (newconn_sid && (flags & OSKIT_SO_SIDREFLECT))
+		return OSKIT_E_INVALIDARG;
+
+	CSID(&csid);
+	STRSOCHECK(csid, s->so, LISTEN, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+
+	if (newconn_sid)
+	{
+	    STRSOCHECK(newconn_sid, s->so, ACCEPT_ASSOCIATE, &rc);
+	    if (rc)
+		    return oskit_freebsd_xlate_errno(rc);
+	    s->so->so_newconn_sid = newconn_sid;
+	    s->so->so_options &= ~SO_SIDREFLECT;
+	}
+	else if (flags & OSKIT_SO_SIDREFLECT)
+	{
+	    s->so->so_newconn_sid = SECSID_NULL;
+	    s->so->so_options |= SO_SIDREFLECT;
+	}
+	
+	return socket_listen(&s->ioi,n);
+}
+
+static OSKIT_COMDECL
+secsocket_getsockname(oskit_socket_secure_t *f, struct oskit_sockaddr *asa,
+		      oskit_size_t *alen,
+		      security_class_t *out_sclass,
+		      security_id_t *out_sid)
+{
+       	oskit_sockimpl_t *s = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+
+	*out_sclass = s->so->so_sclass;
+	*out_sid = s->so->so_sid;
+	return socket_getsockname(&s->ioi,asa,alen);
+}
+
+
+static OSKIT_COMDECL
+secsocket_getpeername(oskit_socket_secure_t *f,
+		      struct oskit_sockaddr *asa, oskit_size_t *alen,
+		      security_id_t *out_peer_sid)
+{
+       	oskit_sockimpl_t *s = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+
+	*out_peer_sid = s->so->so_peer_sid;
+	return socket_getpeername(&s->ioi,asa,alen);
+}
+
+
+static OSKIT_COMDECL
+secsocket_sendto(oskit_socket_secure_t *f, const void *msg, oskit_size_t len,
+		 oskit_u32_t flags, const struct oskit_sockaddr *to,
+		 oskit_size_t tolen, oskit_size_t *retval,
+		 security_id_t dso_sid, security_id_t msg_sid)
+{
+       	oskit_sockimpl_t *s = (oskit_sockimpl_t*) ((char *) f - 
+					   offsetof(oskit_sockimpl_t, sosi));
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, s->so, SEND, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+
+	COMMONSOCHECK(msg_sid, s->so, SEND_ASSOCIATE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);	
+
+	return xlaterc(bsdnet_sendto_secure(s->so,
+				     msg, len, flags,
+				     (struct sockaddr *)to, tolen, retval,
+					    dso_sid, msg_sid));
+}
+
+
+static OSKIT_COMDECL
+secsocket_recvfrom(oskit_socket_secure_t *s, void *buf,
+		   oskit_size_t len, oskit_u32_t flags,
+		   struct oskit_sockaddr *from, oskit_size_t *fromlen,
+		   oskit_size_t *retval, 
+		   security_id_t *out_sso_sid,
+		   security_id_t *out_msg_sid)
+{
+	oskit_sockimpl_t *b = (oskit_sockimpl_t*) ((char *) s - 
+					   offsetof(oskit_sockimpl_t, sosi));
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, b->so, RECEIVE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+
+	return xlaterc(bsdnet_recvfrom_secure(b->so,
+				       buf, len, flags,
+				       (struct sockaddr *)from, fromlen,
+				       retval,out_sso_sid,out_msg_sid));
+}
+#endif
+
 /*******************************************************/
 /******* Implementation of the oskit_stream_t if ********/
 /*******************************************************/
@@ -396,6 +732,15 @@ opensocket_read(oskit_stream_t *f, void *buf, oskit_u32_t len,
 				oskit_u32_t *out_actual)
 {
 	oskit_sockimpl_t *s = (oskit_sockimpl_t *)(f-1);
+#if FLASK
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, s->so, RECEIVE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return xlaterc(bsdnet_read(s->so, buf, len, out_actual));
 }
 
@@ -404,6 +749,15 @@ opensocket_write(oskit_stream_t *f, const void *buf,
 				 oskit_u32_t len, oskit_u32_t *out_actual)
 {
 	oskit_sockimpl_t *s = (oskit_sockimpl_t *)(f-1);
+#if FLASK
+        security_id_t csid;
+	int rc;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, s->so, SEND, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
 	return xlaterc(bsdnet_write(s->so, buf, len, out_actual));
 }
 
@@ -529,7 +883,7 @@ asyncio_add_listener(oskit_asyncio_t *f, struct oskit_listener *l,
 	struct proc 	p;
         register int    s;
 
-	OSKIT_FREEBSD_CREATE_CURPROC(p)
+	OSKIT_FREEBSD_CREATE_CURPROC(p);
         s = splnet();
 	cond = get_socket_conditions(so);
 
@@ -553,7 +907,7 @@ asyncio_add_listener(oskit_asyncio_t *f, struct oskit_listener *l,
 		so->so_snd.sb_flags |= SB_SEL;
 	}
         splx(s);
-	OSKIT_FREEBSD_DESTROY_CURPROC(p)
+	OSKIT_FREEBSD_DESTROY_CURPROC(p);
         return (cond);
 }
 
@@ -627,7 +981,7 @@ bufio_stream_read(oskit_bufio_stream_t *f,
 	OSKIT_FREEBSD_CREATE_CURPROC(p);
 	auio.uio_resid = mlen;
 	auio.uio_procp = &p;
-	rc = soreceive(so, (struct mbuf **)0,
+	rc = so->so_proto->pr_usrreqs->pru_soreceive(so, (struct sockaddr **)0,
 			&auio, (struct mbuf **)&m,
 			(struct mbuf **)0, (int *)0);
 
@@ -656,6 +1010,15 @@ bufio_stream_write(oskit_bufio_stream_t *f,
 	oskit_size_t  payload;
 	void	      *data;
 
+#if FLASK
+        security_id_t csid;
+	
+	CSID(&csid);
+	COMMONSOCHECK(csid, so, SEND, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
+
 	OSKIT_FREEBSD_CREATE_CURPROC(p);
 
 	rc = oskit_bufio_getsize(buf, &size);
@@ -690,8 +1053,8 @@ bufio_stream_write(oskit_bufio_stream_t *f,
         m->m_ext.ext_bufio = buf;
         m->m_flags |= M_EXT;
 
-	rc = sosend(so, (struct mbuf *)0, (struct uio *)0,
-			m, (struct mbuf *)0, 0);
+	rc = so->so_proto->pr_usrreqs->pru_sosend(so, (struct sockaddr *)0, (struct uio *)0,
+			m, (struct mbuf *)0, 0, &p);
 
 	/* unmap ??? */
 bailout:
@@ -726,6 +1089,22 @@ static struct oskit_socket_ops sockops =
 	socket_recvmsg
 };
 
+#if FLASK
+static struct oskit_socket_secure_ops secsockops =
+{
+	secsocket_query,
+	secsocket_addref,
+	secsocket_release,
+	secsocket_accept,
+	secsocket_connect,
+	secsocket_listen,
+	secsocket_getsockname,
+	secsocket_getpeername,
+	secsocket_sendto,
+	secsocket_recvfrom
+};
+#endif
+
 static struct oskit_stream_ops opensockops =
 {
 	opensocket_query,
@@ -780,6 +1159,9 @@ create_sockimpl(struct socket *so)
         si->ios.ops = &opensockops;
         si->ioa.ops = &asyncioops;
         si->iob.ops = &bufio_streamops;
+#if FLASK
+	si->sosi.ops = &secsockops;
+#endif	
 
 	si->so = so;
 	si->readers = oskit_create_listener_mgr((oskit_iunknown_t *)&si->ioa);
@@ -787,6 +1169,7 @@ create_sockimpl(struct socket *so)
 	return si;
 }
 
+
 /***************************************************************************/
 /*
  * methods of socket_factories
@@ -800,6 +1183,7 @@ socket_factory_query(oskit_socket_factory_t *b,
 		*out_ihandle = b;
 		return 0;
 	}
+
 	*out_ihandle = NULL;
 	return OSKIT_E_NOINTERFACE;
 }
@@ -816,6 +1200,92 @@ socket_factory_release(oskit_socket_factory_t *b)
 	return 1;
 }
 
+#if FLASK
+#include <netinet/in.h>
+
+typedef struct 
+{
+    oskit_u32_t domain;
+    oskit_u32_t type;
+    oskit_u32_t protocol;
+    security_class_t sclass;
+} so_sclass_t;
+
+static so_sclass_t so_sclass[] = 
+{
+{ PF_INET, SOCK_STREAM, 0, SECCLASS_IN_STREAM_SOCK },
+{ PF_INET, SOCK_DGRAM, 0, SECCLASS_IN_DGRAM_SOCK },
+{ PF_INET, SOCK_RAW, IPPROTO_ICMP, SECCLASS_IN_ICMP_SOCK },
+{ PF_INET, SOCK_RAW, IPPROTO_IGMP, SECCLASS_IN_IGMP_SOCK },
+{ PF_INET, SOCK_RAW, IPPROTO_RSVP, SECCLASS_IN_RSVP_SOCK },
+{ PF_INET, SOCK_RAW, IPPROTO_IPIP, SECCLASS_IN_IPIP_SOCK },
+{ PF_INET, SOCK_RAW, 0, SECCLASS_IN_RAW_SOCK },
+{ PF_ROUTE, SOCK_RAW, 0, SECCLASS_RTSOCK },
+};
+
+#define NSOSCLASS (sizeof so_sclass)/sizeof(so_sclass_t)
+#endif
+
+
+static oskit_error_t socket_factory_create_helper(oskit_u32_t domain,
+                                                  oskit_u32_t type,
+                                                  oskit_u32_t protocol, 
+#if FLASK
+                                                  security_id_t sid,
+#endif  
+                                                  oskit_socket_t **aso)
+{
+	oskit_error_t   rc;
+        oskit_sockimpl_t *b;
+	struct socket  *so;
+	struct proc     p;
+#if FLASK
+	int index;
+	security_id_t csid;
+	
+	CSID(&csid);
+
+	for (index = 0; index < NSOSCLASS; index++)
+	{
+	    if (domain == so_sclass[index].domain &&
+		type == so_sclass[index].type &&
+		(protocol == so_sclass[index].protocol ||
+		 so_sclass[index].protocol == 0))
+		    break;
+	}
+	if (index >= NSOSCLASS)
+	{
+	    osenv_panic("no socket class for (d %d, t %d, p %d)\n",
+			domain, type, protocol);
+	}
+
+	COMMONSOSIDCHECK(csid, 
+			 so_sclass[index].sclass, (sid ? sid : csid),
+			 CREATE, &rc);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#endif
+	OSKIT_FREEBSD_CREATE_CURPROC(p);
+        rc = socreate(domain, &so, type, protocol, &p);
+	OSKIT_FREEBSD_DESTROY_CURPROC(p);
+	if (rc)
+		return oskit_freebsd_xlate_errno(rc);
+#if FLASK
+	so->so_sclass = so_sclass[index].sclass;
+	so->so_sid = sid ? sid : csid;
+	so->so_newconn_sid = so->so_sid;
+	so->so_peer_sid = SECSID_NULL;
+	osenv_log(OSENV_LOG_DEBUG, 
+		"socket_factory_create():  new socket (sclass %d, sid %d)\n", 
+		  so->so_sclass,
+		  so->so_sid);
+#endif
+
+	b = create_sockimpl(so);
+	*aso = &b->ioi;
+        return 0;    
+}
+
 /*
  * function to create a socket - analog to socket(2)
  */
@@ -825,20 +1295,11 @@ socket_factory_create(
 	oskit_u32_t domain,
 	oskit_u32_t type, oskit_u32_t protocol, oskit_socket_t **aso)
 {
-	oskit_error_t   rc;
-        oskit_sockimpl_t *b;
-	struct socket  *so;
-	struct proc     p;
-
-	OSKIT_FREEBSD_CREATE_CURPROC(p)
-        rc = socreate(domain, &so, type, protocol);
-	OSKIT_FREEBSD_DESTROY_CURPROC(p)
-	if (rc)
-		return oskit_freebsd_xlate_errno(rc);
-
-	b = create_sockimpl(so);
-	*aso = &b->ioi;
-        return 0;
+        return socket_factory_create_helper(domain,type,protocol,
+#if FLASK
+					    SECSID_NULL,
+#endif
+					    aso);
 }
 
 /*
@@ -849,7 +1310,7 @@ socket_factory_create_pair(
 	oskit_socket_factory_t *factory,
 	oskit_u32_t domain,
 	oskit_u32_t type, oskit_u32_t protocol,
-	oskit_socket_t **aso1, oskit_socket_t *aso2)
+	oskit_socket_t **aso1, oskit_socket_t **aso2)
 {
 	return OSKIT_E_NOTIMPL;	/* XXX */
 }
@@ -864,3 +1325,73 @@ static struct oskit_socket_factory_ops sf_ops = {
 };
 
 struct oskit_socket_factory oskit_freebsd_net_socket_factory = { &sf_ops };
+
+
+#if FLASK
+/***************************************************************************/
+/*
+ * methods of socket_factory_secure
+ */
+static OSKIT_COMDECL
+socket_factory_secure_query(oskit_socket_factory_secure_t *b,
+	const struct oskit_guid *iid, void **out_ihandle)
+{
+        if (memcmp(iid, &oskit_iunknown_iid, sizeof(*iid)) == 0 ||
+            memcmp(iid, &oskit_socket_factory_secure_iid, sizeof(*iid)) == 0) {
+		*out_ihandle = b;
+		return 0;
+	}
+
+	*out_ihandle = NULL;
+	return OSKIT_E_NOINTERFACE;
+}
+
+static OSKIT_COMDECL_U
+socket_factory_secure_addref(oskit_socket_factory_secure_t *b)
+{
+	return 1;
+}
+
+static OSKIT_COMDECL_U
+socket_factory_secure_release(oskit_socket_factory_secure_t *b)
+{
+	return 1;
+}
+
+
+static OSKIT_COMDECL
+socket_factory_secure_create(
+	oskit_socket_factory_secure_t *factory,
+	oskit_u32_t domain,
+	oskit_u32_t type, oskit_u32_t protocol, security_id_t sid,
+        oskit_socket_t **aso)
+{
+        return socket_factory_create_helper(domain,type,protocol,
+					    sid,
+					    aso);
+}
+
+
+static OSKIT_COMDECL
+socket_factory_secure_create_pair(
+	oskit_socket_factory_secure_t *factory,
+	oskit_u32_t domain,
+	oskit_u32_t type, oskit_u32_t protocol,
+        security_id_t so1_sid, security_id_t so2_sid,
+	oskit_socket_t **aso1, oskit_socket_t **aso2)
+{
+	return OSKIT_E_NOTIMPL;	/* XXX */
+}
+
+
+static struct oskit_socket_factory_secure_ops secsf_ops = {
+	socket_factory_secure_query,
+	socket_factory_secure_addref,
+	socket_factory_secure_release,
+	socket_factory_secure_create,
+	socket_factory_secure_create_pair
+};
+
+struct oskit_socket_factory_secure oskit_freebsd_net_socket_factory_secure = { &secsf_ops };
+
+#endif
