@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -15,12 +15,12 @@
  * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
  */
 
-#include <malloc.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include <oskit/lmm.h>
 #include <oskit/x86/base_paging.h>
+#include <oskit/x86/pc/phys_lmm.h>
 
 int ptab_alloc(oskit_addr_t *out_ptab_pa)
 {
