@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -15,14 +15,18 @@
  * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
  */
 
-#include "malloc.h"
-#include "fd.h"
-
 /*
  * Initialize the C library.
  */
 void
 oskit_init_libc(void)
 {
-	mem_lock_init();
+	static int	didit = 0;
+
+	if (didit)
+		return;
+	
+	oskit_posixlib_init();
+
+	didit = 1;
 }
