@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -18,7 +18,6 @@
 #include <oskit/c/stdlib.h>
 #include <oskit/c/fd.h>
 #include <oskit/c/fs.h>
-#include <oskit/fs/bmodfs.h>
 
 #ifdef GPROF
 
@@ -32,12 +31,11 @@ extern unsigned long etext;
 void
 base_gprof_init()
 {
-	fd_set_console();
-	fs_init(oskit_bmod_init());
-		
-	atexit(gprof_atexit);
-	atexit(_mcleanup);
-	monstartup((unsigned long)&_start, (unsigned long)&etext);
+	/*
+	 * Stub this for now. This stuff happens to early (before the
+	 * clientos stuff is ready and malloc will work proprely).
+	 *
+	 * Instead, see start_gprof() in the startup library.
+	 */
 }
-
 #endif
