@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1995-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -19,6 +19,8 @@
 
 void *malloc(size_t size)
 {
-	return mallocf(size, 0);
+	CHECK_MEMORY_OBJECT();
+
+	return oskit_mem_alloc(libc_memory_object, size, OSKIT_MEM_AUTO_SIZE);
 }
 
