@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -15,54 +15,25 @@
  * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
  */
 /*
- * This defines the oskit's default memory locking for multi-threaded
- * malloc. 
+ * The Minimal C library version of mem_lock does nothing. You need to
+ * use the FreeBSD C library if you want to multithread an oskit kernel.
+ *
+ * Note that you need to arrange for the kernel version of these functions
+ * to be linked in if your program calls malloc out of device drivers (as
+ * the network drivers are want to do).
  */
 
 #include "malloc.h"
 #include "stdlib.h"
-#include <oskit/com/services.h>
-#include <oskit/com/lock_mgr.h>
-#include <oskit/com/lock.h>
-
-static oskit_lock_t	*memlock;
 
 void mem_unlock(void)
 {
-	if (!memlock)
-		return;
-	
-	oskit_lock_unlock(memlock);
 }
 
 void mem_lock(void)
 {
-	if (!memlock)
-		return;
-	
-	oskit_lock_lock(memlock);
 }
 
-/*
- * Query the registry for the lock_mgr. If its null, then no memory locking
- * is needed. Otherwise create a lock.
- */
-void mem_lock_init()
+void mem_lock_init(void)
 {
-	oskit_lock_mgr_t	*lock_mgr;
-
-	/*
-	 * Allow only once!
-	 */
-	if (memlock)
-		return;
-
-	if (oskit_lookup_first(&oskit_lock_mgr_iid, (void *) &lock_mgr))
-		panic("mem_lock_init: oskit_lookup_first");
-
-	if (!lock_mgr)
-		return;
-
-	if (oskit_lock_mgr_allocate_critical_lock(lock_mgr, &memlock))
-		panic("mem_lock_init: oskit_lock_mgr_allocate_lock");
 }
