@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1997, 1998 The University of Utah and the Flux Group.
+ * Copyright (c) 1996-1999 The University of Utah and the Flux Group.
  * 
  * This file is part of the OSKit Linux Glue Libraries, which are free
  * software, also known as "open source;" you can redistribute them and/or
@@ -19,7 +19,6 @@
  * and do not make use of the buffer operations vectors.
  */
 
-#include <oskit/dev/dev.h>
 #include <oskit/dev/blk.h>
 #include <oskit/dev/isa.h>
 #include <oskit/dev/native.h>
@@ -37,6 +36,7 @@
 #include "linux_emul.h"
 #include "sched.h"
 #include "shared.h"
+#include "osenv.h"
 
 /*
  * Name to Linux device number mapping table.
@@ -46,22 +46,22 @@ static struct {
 	char	*name;
 	kdev_t	dev;
 } name_to_dev[] = {
-	{ "sd0", MKDEV(SCSI_DISK_MAJOR, 0) },
-	{ "sd1", MKDEV(SCSI_DISK_MAJOR, 1) },
-	{ "sd2", MKDEV(SCSI_DISK_MAJOR, 2) },
-	{ "sd3", MKDEV(SCSI_DISK_MAJOR, 3) },
-	{ "sd4", MKDEV(SCSI_DISK_MAJOR, 4) },
-	{ "sd5", MKDEV(SCSI_DISK_MAJOR, 5) },
-	{ "sd6", MKDEV(SCSI_DISK_MAJOR, 6) },
-	{ "sd7", MKDEV(SCSI_DISK_MAJOR, 7) },
-	{ "sda", MKDEV(SCSI_DISK_MAJOR, 0) },
-	{ "sdb", MKDEV(SCSI_DISK_MAJOR, 1) },
-	{ "sdc", MKDEV(SCSI_DISK_MAJOR, 2) },
-	{ "sdd", MKDEV(SCSI_DISK_MAJOR, 3) },
-	{ "sde", MKDEV(SCSI_DISK_MAJOR, 4) },
-	{ "sdf", MKDEV(SCSI_DISK_MAJOR, 5) },
-	{ "sdg", MKDEV(SCSI_DISK_MAJOR, 6) },
-	{ "sdh", MKDEV(SCSI_DISK_MAJOR, 7) },
+	{ "sd0", MKDEV(SCSI_DISK0_MAJOR, 0) },
+	{ "sd1", MKDEV(SCSI_DISK0_MAJOR, 1) },
+	{ "sd2", MKDEV(SCSI_DISK0_MAJOR, 2) },
+	{ "sd3", MKDEV(SCSI_DISK0_MAJOR, 3) },
+	{ "sd4", MKDEV(SCSI_DISK0_MAJOR, 4) },
+	{ "sd5", MKDEV(SCSI_DISK0_MAJOR, 5) },
+	{ "sd6", MKDEV(SCSI_DISK0_MAJOR, 6) },
+	{ "sd7", MKDEV(SCSI_DISK0_MAJOR, 7) },
+	{ "sda", MKDEV(SCSI_DISK0_MAJOR, 0) },
+	{ "sdb", MKDEV(SCSI_DISK0_MAJOR, 1) },
+	{ "sdc", MKDEV(SCSI_DISK0_MAJOR, 2) },
+	{ "sdd", MKDEV(SCSI_DISK0_MAJOR, 3) },
+	{ "sde", MKDEV(SCSI_DISK0_MAJOR, 4) },
+	{ "sdf", MKDEV(SCSI_DISK0_MAJOR, 5) },
+	{ "sdg", MKDEV(SCSI_DISK0_MAJOR, 6) },
+	{ "sdh", MKDEV(SCSI_DISK0_MAJOR, 7) },
 	{ "wd0", MKDEV(IDE0_MAJOR, 0) },
 	{ "wd1", MKDEV(IDE0_MAJOR, 1) },
 	{ "wd2", MKDEV(IDE1_MAJOR, 0) },
@@ -334,43 +334,22 @@ oskit_linux_block_open_kdev(int kdev, struct gendisk *gd,
 	po->inode.i_rdev = kdev;
 
 	po->inode.i_emul_data = po;
-	if (gd)
-		po->size = gd->part[MINOR(kdev)].nr_sects;
-	else {
-		if (!blk_size[MAJOR(kdev)]
-		    || !blk_size[MAJOR(kdev)][MINOR(kdev)])
-			panic("%s:%d: bad block size", __FILE__, __LINE__);
 
-		po->size = (blk_size[MAJOR(kdev)][MINOR(kdev)]
-			    << (BLOCK_SIZE_BITS - DISK_BLOCK_BITS));
-	}
-
-	if (blksize_size[MAJOR(kdev)]
-	    && blksize_size[MAJOR(kdev)][MINOR(kdev)])
-		po->bsize = blksize_size[MAJOR(kdev)][MINOR(kdev)];
-	else
-		/* XXX: Linux uses BLOCK_SIZE =1024, not 512 by default */
-		po->bsize = DISK_BLOCK_SIZE;
-	po->bmask = po->bsize - 1;
-
-	for (i = DISK_BLOCK_SIZE, po->bshift = DISK_BLOCK_BITS; i < po->bsize;
-	     i <<= ++po->bshift)
-		;
 	po->next = open_list;
 	open_list = po->next;
 
-	if (!po->fops->open) {
-		po->busy = 0;
-		*out_io = &po->ioi;
+	if (!po->fops->open)
 		err = 0;
-		goto finish;
+	else {
+		struct dentry dentry;
+		file.f_mode = mode;
+		file.f_flags = 0;
+		file.f_dentry = &dentry;
+		dentry.d_inode = &po->inode;
+
+		err = (*po->fops->open)(&po->inode, &file);
 	}
 
-	file.f_mode = mode;
-	file.f_flags = 0;
-
-	err = (*po->fops->open)(&po->inode, &file);
-
 	if (err) {
 		open_list_remove(po);
 		if (po->want) {
@@ -381,6 +360,37 @@ oskit_linux_block_open_kdev(int kdev, struct gendisk *gd,
 		oskit_linux_mem_free(po, OSENV_PHYS_WIRED,
 				    sizeof(struct peropen));
 	} else {
+		/*
+		 * Now that the device has been opened,
+		 * we can extract its size.  This might
+		 * not have been set properly in blk_size
+		 * before the open call, e.g. for the floppy driver.
+		 */
+		if (gd)
+			po->size = gd->part[MINOR(kdev)].nr_sects;
+		else {
+			osenv_assert(blk_size[MAJOR(kdev)] &&
+				     blk_size[MAJOR(kdev)][MINOR(kdev)]);
+
+			po->size = (blk_size[MAJOR(kdev)][MINOR(kdev)]
+				    << (BLOCK_SIZE_BITS - DISK_BLOCK_BITS));
+		}
+
+		if (blksize_size[MAJOR(kdev)]
+		    && blksize_size[MAJOR(kdev)][MINOR(kdev)]) {
+			int bshift = DISK_BLOCK_BITS;
+			po->bsize = blksize_size[MAJOR(kdev)][MINOR(kdev)];
+			for (i = po->bsize >> DISK_BLOCK_BITS; i != 1; i >>= 1)
+				bshift++;
+			po->bshift = bshift;
+		}
+		else {
+			/* XXX: Linux uses BLOCK_SIZE=1024, not 512 */
+			po->bsize = DISK_BLOCK_SIZE;
+			po->bshift = DISK_BLOCK_BITS;
+		}
+		po->bmask = po->bsize - 1;
+
 		po->busy = 0;
 		*out_io = &po->ioi;
 		if (po->want) {
@@ -466,9 +476,12 @@ blkio_release(oskit_blkio_t *io)
 	newcount = --po->count;
 	if (newcount == 0) {
 		if (po->fops->release) {
-			po->busy = 1;
+			struct dentry dentry;
 			file.f_mode = po->mode;
 			file.f_flags = 0;
+			file.f_dentry = &dentry;
+			dentry.d_inode = &po->inode;
+			po->busy = 1;
 			(*po->fops->release)(&po->inode, &file);
 		}
 		open_list_remove(po);
@@ -511,6 +524,7 @@ blkio_read(oskit_blkio_t *io, void *buf,
 {
 	int err;
 	struct file file;
+	struct dentry dentry;
 	struct peropen *po;
 	unsigned bn, sz;
 	struct task_struct ts;
@@ -545,7 +559,11 @@ blkio_read(oskit_blkio_t *io, void *buf,
 	file.f_mode = po->mode;
 	file.f_flags = 0;
 	file.f_pos = off;
-	err = (*po->fops->read)(&po->inode, &file, buf, count);
+	file.f_inode = &po->inode;
+	file.f_dentry = &dentry;
+	dentry.d_inode = &po->inode;
+
+	err = (*po->fops->read)(&file, buf, count, 0);
 	if (err) {
 		err = linux_to_oskit_error(err);
 		*amount_read = 0;	/* XXX */
@@ -566,6 +584,7 @@ blkio_write(oskit_blkio_t *io, const void *buf,
 {
 	int err;
 	struct file file;
+	struct dentry dentry;
 	struct peropen *po;
 	unsigned bn, sz;
 	struct task_struct ts;
@@ -600,7 +619,10 @@ blkio_write(oskit_blkio_t *io, const void *buf,
 	file.f_mode = po->mode;
 	file.f_flags = 0;
 	file.f_pos = off;
-	err = (po->fops->write)(&po->inode, &file, buf, count);
+	file.f_inode = &po->inode;
+	file.f_dentry = &dentry;
+	dentry.d_inode = &po->inode;
+	err = (po->fops->write)(&file, buf, count, 0);
 	if (err) {
 		osenv_log(OSENV_LOG_NOTICE, "WRITE error %d, 0x%x\n", err, err);
 		err = linux_to_oskit_error(err);
