@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1997, 1998 The University of Utah and the Flux Group.
+ * Copyright (c) 1996-1999 The University of Utah and the Flux Group.
  * 
  * This file is part of the OSKit Linux Glue Libraries, which are free
  * software, also known as "open source;" you can redistribute them and/or
@@ -20,12 +20,10 @@
 #define OSKIT
 #endif
 
-#include <oskit/dev/dev.h>
-#include <oskit/x86/pc/dev.h>
-
 #include <linux/errno.h>
 #include <linux/sched.h>
 #include <asm/dma.h>
+#include "osenv.h"
 
 /*
  * Allocate a DMA channel.
