@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
+ * Copyright (c) 1997, 1998, 1999 The University of Utah and the Flux Group.
  * 
  * This file is part of the OSKit Linux Glue Libraries, which are free
  * software, also known as "open source;" you can redistribute them and/or
@@ -17,7 +17,6 @@
  */
 
 #include <oskit/types.h>
-#include <oskit/time.h>
 #include <oskit/fs/fs.h>
 #include <assert.h>
 
@@ -29,15 +28,20 @@
 int
 fs_linux_current_time()
 {
-	struct oskit_timespec ts;
-	oskit_error_t err;
+	struct timeval tv, *tvp = &tv;
 	struct task_struct *task;
 
 	task = current;
-	err = fs_gettime(&ts);
-	assert(err == 0);
+	/* Right thing to do? */
+	if (gettimeofday(tvp, 0))
+		panic("fs_linux_current_time: gettimeofday failed");
 	current = task;
 
-	return ts.tv_sec;
+	while (tvp->tv_usec > 1000000) {
+		tvp->tv_sec++;
+		tvp->tv_usec -= 1000000;
+	}
+	
+	return tv.tv_sec;
 }
 
