@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998 The University of Utah and the Flux Group.
+ * Copyright (c) 1997, 1998, 1999 The University of Utah and the Flux Group.
  * 
  * This file is part of the OSKit Linux Glue Libraries, which are free
  * software, also known as "open source;" you can redistribute them and/or
@@ -21,7 +21,7 @@
 
 #include <linux/kernel.h>
 #include <oskit/c/stdarg.h>
-#include <oskit/dev/dev.h>
+#include "osenv.h"
 
 void panic(const char *fmt, ...)
 {
@@ -30,6 +30,9 @@ void panic(const char *fmt, ...)
         va_start(args, fmt);
         osenv_vpanic(fmt, args);
         va_end(vl);
+#ifndef CPUNITS
+        /* This is here to suppress the noreturn warning */
 	exit(1);
+#endif
 }
 
