@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -63,6 +63,16 @@
  * oskit_file and oskit_dir methods; per-node operations
  */
 
+/*
+ * Internal representation of a single dirent entry.
+ */
+typedef struct internal_dirent {
+	oskit_size_t	namelen;		/* length of name */
+	oskit_ino_t	ino;			/* entry inode */
+	char	       *name;			/* entry name */
+} internal_dirent_t;
+static oskit_error_t	fs_dirents_create(internal_dirent_t *entries,
+				int count, oskit_dirents_t **out_dirents);
 
 /*
  * vnode-based helper functions for both oskit_file and oskit_dir methods
@@ -1348,7 +1358,7 @@ static char dirbuf[DEV_BSIZE];
 
 static OSKIT_COMDECL dir_getdirentries(oskit_dir_t *d, oskit_u32_t *inout_ofs,
 				      oskit_u32_t nentries,
-				      struct oskit_dirents *out_dirents)
+				      struct oskit_dirents **out_dirents)
 {
     struct gdir *dir;
     oskit_error_t ferror;
@@ -1356,7 +1366,9 @@ static OSKIT_COMDECL dir_getdirentries(oskit_dir_t *d, oskit_u32_t *inout_ofs,
     struct uio auio;
     struct iovec aiov;
     struct dirent *dp, *edp;
-    int error, eofflag, actual;
+    int error, eofflag, actual, count, dircount;
+    oskit_error_t rc;
+    struct internal_dirent *dirents;
 
     dir = (struct gdir *)d;
     if (!dir || !dir->f.count || !out_dirents || !inout_ofs)
@@ -1374,13 +1386,13 @@ static OSKIT_COMDECL dir_getdirentries(oskit_dir_t *d, oskit_u32_t *inout_ofs,
 	    return errno_to_oskit_error(error);
     }
 
-    out_dirents->count = 0;
-    MALLOC(out_dirents->dirents,struct oskit_dirent*,
-	   sizeof(struct oskit_dirent)*nentries,M_TEMP,M_WAITOK);
-    assert(out_dirents->dirents);
+    dircount = 0;
+    MALLOC(dirents, struct internal_dirent *,
+	   sizeof(struct internal_dirent)*nentries,M_TEMP,M_WAITOK);
+    assert(dirents);
 
     error = 0;
-    while (out_dirents->count < nentries)
+    while (dircount < nentries)
     {
 	aiov.iov_base = dirbuf;
 	aiov.iov_len = sizeof(dirbuf);
@@ -1402,37 +1414,71 @@ static OSKIT_COMDECL dir_getdirentries(oskit_dir_t *d, oskit_u32_t *inout_ofs,
 		/* error or end-of-file */
 		break;
 
+	/*
+	 * First count them up in case we need to make the dirents
+	 * array bigger.
+	 */
+	count = 0;
 	edp = (struct dirent *)&dirbuf[actual];
 	for (dp = (struct dirent *)dirbuf; dp < edp; )
 	{
 	    if (dp->d_reclen <= 0)
 		    break;
 
-	    out_dirents->count++;
+	    count++;
+	    
+	    dp = (struct dirent *) ((char *)dp + dp->d_reclen);
+	}
 
-	    if (out_dirents->count > nentries)
-	    {
-		out_dirents->dirents =
-			(struct oskit_dirent *)
-				realloc(out_dirents->dirents,
-					sizeof(struct oskit_dirent)*
-					out_dirents->count);
-		assert(out_dirents->dirents);
-	    }
+	/*
+	 * If there are too many, increase the dirents array in one chunk.
+	 */
+	if (dircount + count > nentries)
+	{
+	    int newcount = dircount + count;
+	    struct internal_dirent *ptmp;
 
-	    MALLOC(out_dirents->dirents[out_dirents->count-1].name,
+	    MALLOC(ptmp, struct internal_dirent *,
+		   sizeof(struct internal_dirent)*newcount, M_TEMP, M_WAITOK);
+	    assert(ptmp);
+
+	    memcpy(ptmp, dirents,
+		   sizeof(struct internal_dirent) * dircount);
+
+	    FREE(dirents, M_TEMP);
+	    dirents = ptmp;
+	}
+
+	/*
+	 * Finally, copy them out.
+	 */
+	edp = (struct dirent *)&dirbuf[actual];
+	for (dp = (struct dirent *)dirbuf; dp < edp; )
+	{
+	    if (dp->d_reclen <= 0)
+		    break;
+
+	    dircount++;
+
+	    MALLOC(dirents[dircount-1].name,
 		   char *, sizeof(char)*(dp->d_namlen+1),
 		   M_TEMP, M_WAITOK);
-	    assert(out_dirents->dirents[out_dirents->count-1].name);
-	    strncpy(out_dirents->dirents[out_dirents->count-1].name,
-		    dp->d_name,dp->d_namlen+1);
+	    assert(dirents[dircount-1].name);
+	    strncpy(dirents[dircount-1].name, dp->d_name,dp->d_namlen+1);
 
-	    out_dirents->dirents[out_dirents->count-1].ino = dp->d_fileno;
+	    dirents[dircount-1].ino = dp->d_fileno;
+	    dirents[dircount-1].namelen = dp->d_namlen;
 
 	    dp = (struct dirent *) ((char *)dp + dp->d_reclen);
 	}
     }
 
+    /*
+     * Create the dirents COM object first to make sure we can.
+     */
+    rc = fs_dirents_create(dirents, dircount, out_dirents);
+    assert(!rc);
+
     prfree(p);
     return errno_to_oskit_error(error);
 }
@@ -1575,6 +1621,149 @@ static struct oskit_dir_ops dir_ops = {
     dir_reparent
 };
 
+/*
+ * oskit_dirents_t COM object implementation.
+ */
+struct fs_dirents {
+	oskit_dirents_t		direntsi;	/* COM dirents interface */
+	int			count;		/* Reference count */
+	internal_dirent_t       *entries;	/* Array of entries */
+	int			dircount;	/* Number of entries */
+	int			index;		/* Current index pointer */
+};
+
+/*
+ * Methods.
+ */
+static struct oskit_dirents_ops fs_dirents_ops;
+
+static OSKIT_COMDECL
+fs_dirents_query(oskit_dirents_t *d,
+		 const oskit_iid_t *iid, void **out_ihandle)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	assert(fsdirents->count);
+
+        if (memcmp(iid, &oskit_iunknown_iid, sizeof(*iid)) == 0 ||
+            memcmp(iid, &oskit_dirents_iid, sizeof(*iid)) == 0) {
+                *out_ihandle = fsdirents;
+                ++fsdirents->count;
+                return 0;
+        }
+
+        *out_ihandle = NULL;
+        return OSKIT_E_NOINTERFACE;
+}
+
+static OSKIT_COMDECL_U
+fs_dirents_addref(oskit_dirents_t *d)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	assert(fsdirents->count);
+	
+	return ++fsdirents->count;
+}
+
+static OSKIT_COMDECL_U
+fs_dirents_release(oskit_dirents_t *d)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	int i;
+	
+	assert(fsdirents->count);
+
+	if (--fsdirents->count)
+		return fsdirents->count;
+
+	for (i = 0; i < fsdirents->dircount; i++)
+		FREE(fsdirents->entries[i].name, M_TEMP);
+
+	FREE(fsdirents->entries, M_TEMP);
+	FREE(fsdirents, M_TEMP);	
+	return 0;
+}
+
+static OSKIT_COMDECL
+fs_dirents_getcount(oskit_dirents_t *d, int *out_count)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	assert(fsdirents->count);
+
+	*out_count = fsdirents->dircount;
+
+	return 0;
+}
+
+static OSKIT_COMDECL
+fs_dirents_getnext(oskit_dirents_t *d, oskit_dirent_t *dirent)
+{
+	struct fs_dirents	*fsdirents = (struct fs_dirents *) d;
+	struct internal_dirent  *idirent;
+	
+	assert(fsdirents->count);
+
+	if (fsdirents->index >= fsdirents->dircount)
+		return OSKIT_EWOULDBLOCK;
+
+	idirent = &fsdirents->entries[fsdirents->index];
+
+	if (dirent->namelen < idirent->namelen + 1)
+		return OSKIT_ENAMETOOLONG;
+
+	dirent->ino     = idirent->ino;
+	dirent->namelen = idirent->namelen;
+	strcpy(dirent->name, idirent->name);
+	
+	fsdirents->index++;	
+	return 0;
+}
+
+static OSKIT_COMDECL
+fs_dirents_rewind(oskit_dirents_t *d)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	assert(fsdirents->count);
+
+	fsdirents->index = 0;
+	return 0;
+}
+
+static struct oskit_dirents_ops fs_dirents_ops = {
+	fs_dirents_query,
+	fs_dirents_addref,
+	fs_dirents_release,
+	fs_dirents_getcount,
+	fs_dirents_getnext,
+	fs_dirents_rewind,
+};
+
+/*
+ * Create a oskit_dirents_t COM object from an array of oskit_dirent_t
+ * entries.
+ */
+static oskit_error_t
+fs_dirents_create(internal_dirent_t *entries,
+		  int count, oskit_dirents_t **out_dirents)
+{
+        struct fs_dirents *fsdirents;
+ 
+	MALLOC(fsdirents, struct fs_dirents *,
+	       sizeof(struct fs_dirents),M_TEMP,M_WAITOK);
+
+        if (fsdirents == NULL)
+                return OSKIT_ENOMEM;
+
+        fsdirents->count        = 1;
+        fsdirents->dircount     = count;
+        fsdirents->entries      = entries;
+        fsdirents->index        = 0;
+	fsdirents->direntsi.ops = &fs_dirents_ops;
+
+	*out_dirents = &fsdirents->direntsi;
+
+	return 0;
+}
+
 #if 0
 /* This is just for debugging. */
 oskit_error_t
