@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -21,10 +21,13 @@
 #include <oskit/io/blkio.h>
 #include <oskit/fs/filesystem.h>
 #include <oskit/fs/dir.h>
+#include <oskit/dev/osenv.h>
 
-oskit_error_t fs_netbsd_init(void);
+oskit_error_t fs_netbsd_init(oskit_osenv_t *osenv);
 
 oskit_error_t fs_netbsd_mount(oskit_blkio_t *bio, oskit_u32_t flags,
 			      oskit_filesystem_t **out_fs);
 
+oskit_error_t fs_netbsd_threaded_init(oskit_osenv_t *osenv);
+
 #endif _OSKIT_FS_NETBSD_H_
