@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -24,26 +24,7 @@
 int
 chroot(const char *path)
 {
-	oskit_file_t *f;
-	oskit_error_t rc;
+	oskit_error_t	rc = fs_chroot(path);
 
-	rc = fs_lookup(path, &f);
-	if (!rc) {
-		oskit_dir_t *d;
-		rc = oskit_file_query(f, &oskit_dir_iid, (void **)&d);
-		oskit_file_release(f);
-		if (!rc) {
-			oskit_dir_t *rooted;
-			rc = oskit_dir_reparent(d, NULL, &rooted);
-			oskit_dir_release(d);
-			if (!rc) {
-				if (fs_root) /* XXX locking? */
-					oskit_dir_release(fs_root);
-				fs_root = rooted;
-			}
-		}
-		else if (rc == OSKIT_E_NOINTERFACE)
-			rc = OSKIT_ENOTDIR;
-	}
 	return rc ? (errno = rc), -1 : 0;
 }
