@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -16,8 +16,9 @@
  */
 
 #include <unistd.h>
-#include "fd.h"
-#include "fs.h"
+#include <fs.h>
+#include <fd.h>
+#include "posix.h"
 
 int
 fchdir(int fd)
@@ -45,9 +46,10 @@ fchdir(int fd)
 		return -1;
 	}
 
-	if (fs_cwd)
-		oskit_dir_release(fs_cwd);
-	fs_cwd = d;
-	return 0;
+	rc = fs_chdir(d);
+	if (!rc)
+		oskit_dir_release(d);
+	
+	return rc;
 }
 
