@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -15,10 +15,14 @@
  * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
  */
 
-#include <unistd.h>
+#include <stdio.h>
+#include <errno.h>
 
 int
 getdirentries(int fd, char *buf, int nbytes, long *basep)
 {
-	panic("getdirentries: Not yet, no how, no way!");
+	printf("getdirentries: Not yet, no how, no way!\n");
+
+	errno = ENOSYS;
+	return -1;
 }
