@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -24,26 +24,21 @@
 int
 lchown(const char *name, oskit_uid_t uid, oskit_gid_t gid)
 {
-        oskit_dir_t *dir;
         oskit_file_t *f;
-        oskit_error_t rc = fs_lookup_dir(&name, &dir);
+        oskit_error_t rc = fs_lookup(name, FSLOOKUP_NOFOLLOW, &f);
 
         if (!rc) {
-                rc = oskit_dir_lookup(dir, name, &f);
-		if (!rc) {
-			struct oskit_stat sb;
-			oskit_u32_t mask = 0;
-			sb.uid = uid;
-			sb.gid = gid;
-			if (uid != -1) 
-				mask |= OSKIT_STAT_UID;
-			if (gid != -1) 
-				mask |= OSKIT_STAT_GID;
+		struct oskit_stat sb;
+		oskit_u32_t mask = 0;
+		sb.uid = uid;
+		sb.gid = gid;
+		if (uid != -1) 
+			mask |= OSKIT_STAT_UID;
+		if (gid != -1) 
+			mask |= OSKIT_STAT_GID;
 			
-			rc = oskit_file_setstat(f, mask, &sb);
-			oskit_file_release(f);
-		}
-		oskit_dir_release(dir);
+		rc = oskit_file_setstat(f, mask, &sb);
+		oskit_file_release(f);
 	}
 	return rc ? (errno = rc), -1 : 0;
 }
