@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -24,7 +24,7 @@ long int
 pathconf(const char *name, int confname)
 {
 	oskit_file_t *f;
-	oskit_error_t rc = fs_lookup(name, &f);
+	oskit_error_t rc = fs_lookup(name, FSLOOKUP_FOLLOW, &f);
 
 	if (!rc) {
 		oskit_s32_t val;
