@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -29,17 +29,12 @@ readlink(const char *name, void *buf, size_t size)
 	oskit_error_t rc;
 	oskit_u32_t actual;
 
-	if ((rc = fs_lookup(name, &f)) != 0) {
-		errno = rc;
-		return -1;
-	}
+	rc = fs_lookup(name, FSLOOKUP_NOFOLLOW, &f);
 
-	rc = oskit_file_readlink(f, buf, size, &actual);
-	oskit_file_release(f);
-	if (rc) {
-		errno = rc;
-		return -1;
+	if (!rc) {
+		rc = oskit_file_readlink(f, buf, size, &actual);
+		oskit_file_release(f);
 	}
-	return actual;
+	return rc ? (errno = rc), -1 : actual;
 }
 
