@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -25,7 +25,7 @@ int
 rmdir(const char *name)
 {
 	oskit_dir_t *dir;
-	oskit_error_t rc = fs_lookup_dir(&name, &dir);
+	oskit_error_t rc = fs_lookup_dir(&name, FSLOOKUP_NOFLAGS, &dir);
 
 	if (!rc) {
 		rc = oskit_dir_rmdir(dir, name);
