@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -25,7 +25,7 @@ int
 symlink(const char *path1, const char *path2)
 {
 	oskit_dir_t *d2;
-	oskit_error_t rc = fs_lookup_dir(&path2, &d2);
+	oskit_error_t rc = fs_lookup_dir(&path2, FSLOOKUP_NOFLAGS, &d2);
 
 	if (!rc) {
 		rc = oskit_dir_symlink(d2, path2, path1);
