@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -19,14 +19,20 @@
 /*
  * Full Blown CPU Inheritance scheduler.
  */
+
+#ifdef  SMP
+Sorry, this code does not work in SMP mode. No locking at all!
+#endif
+
 #include <threads/pthread_internal.h>
+#include "pthread_signal.h"
 #include <stdlib.h>
 
 /*
  * There is a single root scheduler.
  */
 pthread_thread_t        *pthread_root_scheduler = NULL_THREADPTR;
-int			 cpuinherit_debug       = CPUDEBUG_TRUE;
+int			 cpuinherit_debug       = 0;
 
 
 #ifdef	MEASURE
@@ -74,13 +80,9 @@ donorq_remove(queue_head_t *q, pthread_thread_t *pthread)
  */
 char *msg_sched_typenames[] = {
 	"BADTYPE",
-	"BLOCK",
 	"UNBLOCK",
 	"SETSTATE",
 	"EXITED",
-	"USERYIELD",
-	"YIELD",
-	"PREEMPT",
 	"NEWTHREAD",
 	"MAXTYPE",
 };
@@ -123,7 +125,8 @@ pthread_init_scheduler(void)
 	if ((bp = getenv("CPUDEBUG")) != NULL)
 		cpuinherit_debug = strtol(bp, NULL, 0);
 
-	printf("CPUINHERIT_DEBUG = 0x%x\n", cpuinherit_debug);
+	if (cpuinherit_debug)
+	    printf("CPUINHERIT_DEBUG = 0x%x\n", cpuinherit_debug);
 #ifdef MEASURE
 	atexit(dump_scheduler_stats);
 #endif
@@ -181,16 +184,21 @@ void
 pthread_sched_switchto(pthread_thread_t *pthread)
 {
 	pthread_thread_t	*cur = CURPTHREAD();
-	int			s;
-	
-	s = splhigh();
-	pthread_lock(&pthread->lock);
-	pthread_lock(&cur->lock);
-	thread_switch(pthread, &cur->lock, &CURPTHREAD());
+	int			enabled, preemptable;
 
-	/* Control returns from switch with current thread lock locked */
-	pthread_unlock(&cur->lock);
-	splx(s);
+	save_preemption_enable(preemptable);
+	save_interrupt_enable(enabled);
+	disable_interrupts();
+	
+	/* Don't worry about the lock */
+	thread_switch(pthread, &cur->schedlock, CURPTHREAD());
+
+	/* Must set the current thread pointer! */
+	SETCURPTHREAD(cur);
+
+	restore_interrupt_enable(enabled);
+	/* Need to really *restore* the flag since we are in a new context */
+	PREEMPT_ENABLE = preemptable;
 }
 
 /*
@@ -202,11 +210,13 @@ pthread_sched_dispatch(resched_flags_t reason)
 {
 	pthread_thread_t	*pthread = CURPTHREAD();
 	pthread_thread_t	*inheritor, *donator, *nextup;
-	int			s, special = 0;
+	int			enabled, preemptable;
 #ifdef  MEASURE
 	unsigned long		before, after;
 #endif	
-	s = splhigh();
+	save_preemption_enable(preemptable);
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 
 #ifdef  MEASURE
 	before = get_tsc();
@@ -236,9 +246,6 @@ pthread_sched_dispatch(resched_flags_t reason)
 	else
 		pthread->schedflags = SCHED_READY;
 
-	if (nextup)
-		foobar();
-
 	/*
 	 * Go back through the inheritance links, stopping at the first
 	 * thread to indicate that it wants to be woken up. That thread
@@ -246,8 +253,6 @@ pthread_sched_dispatch(resched_flags_t reason)
 	 * unlocking itself when it gets to the other side.
 	 */
 	while (donator) {
-		pthread_lock(&donator->lock);
-
 		switch (reason) {
 		case RESCHED_USERYIELD:
 			donator->donate_rc = SCHEDULE_YIELDED;
@@ -289,7 +294,6 @@ pthread_sched_dispatch(resched_flags_t reason)
 
 		inheritor = donator;
 		donator   = donator->inherits_from;
-		pthread_unlock(&inheritor->lock);
 	}
 	
 	if (donator == NULL_THREADPTR) {
@@ -301,8 +305,7 @@ pthread_sched_dispatch(resched_flags_t reason)
 		 * do.
 		 */
 		if ((donator = IDLETHREAD) == pthread) {
-			pthread_unlock(&pthread->lock);
-			splx(s);
+			restore_interrupt_enable(enabled);
 			return 0;
 		}
 
@@ -312,7 +315,6 @@ pthread_sched_dispatch(resched_flags_t reason)
 		 * so don't want to terminate its message wait when the
 		 * idle loop does its yield to look for a new thread.
 		 */
-		pthread_lock(&donator->lock);
 		donator->inherits_from = NULL_THREADPTR;
 		donator->nextup        = NULL_THREADPTR;
 		
@@ -353,7 +355,7 @@ dispatch:
 	/*
 	 * Clear directed yield flag.
 	 */
-	pthread->preempt = 0;
+	PREEMPT_NEEDED = pthread->preempt = 0;
 	assert(donator);
 
 #ifdef  MEASURE
@@ -363,15 +365,18 @@ dispatch:
 		stats.switch_cycles += (after - before);
 	}
 #endif
-	thread_switch(donator, &pthread->lock, &CURPTHREAD());
+	/* Don't worry about the lock */
+	thread_switch(donator, &pthread->schedlock, CURPTHREAD());
+
+	/* Must set the current thread pointer! */
+	SETCURPTHREAD(pthread);
 
 	assert(pthread->inherits_from ||
 	       ((pthread == pthread_root_scheduler) ||
 		(pthread == IDLETHREAD)));
 	
 	/*
-	 * Thread has switched back in. Note from above that the lock for
-	 * target thread is held across the switch.
+	 * Thread has switched back in. 
 	 *
 	 * Time to look for exceptions before letting it return to
 	 * whatever it was doing before it yielded.
@@ -384,7 +389,7 @@ dispatch:
 		 * be allowed to return, even if it was killed, so the driver
 		 * state can be cleaned up. The death will be noticed later.
 		 */
-		;
+		goto done;
 	}
 	else if ((pthread->flags &
 		  (THREAD_KILLED|THREAD_EXITING)) == THREAD_KILLED) {
@@ -405,10 +410,19 @@ dispatch:
 			 * Never returns
 			 */
 		}
+		else
+			goto done;
 	}
+	
+	/*
+	 * Look for signals. Deliver them in the context of the thread.
+	 */
+	SIGCHECK(pthread);
 
-	pthread_unlock(&pthread->lock);
-	splx(s);
+   done:
+	restore_interrupt_enable(enabled);
+	/* Need to really *restore* the flag since we are in a new context */
+	PREEMPT_ENABLE = preemptable;
 	return 1;
 }
 
@@ -418,27 +432,21 @@ dispatch:
 int
 pthread_sched_reschedule(resched_flags_t reason, pthread_lock_t *plock)
 {
-	pthread_thread_t	*pthread = CURPTHREAD();
-	int			s, rc;
+	int			enabled, rc;
 
-	s = splhigh();
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 
 	/*
-	 * Need to lock the thread lock and then release the provided
-	 * spinlock. This provides atomicity with regards to the thread
-	 * that is switching out since in all cases the lock is either
-	 * the thread lock, or a queue that it just put itself on.
+	 * No thread locking, but need to release the provided lock
+	 * in case it was built with locks enabled.
 	 */
-	if (plock != &(pthread->lock)) {
-		pthread_lock(&(pthread->lock));
-		
-		if (plock)
-			pthread_unlock(plock);
-	}
-
+	if (plock)
+		pthread_unlock(plock);
+	
 	rc = pthread_sched_dispatch(reason);
 
-	splx(s);
+	restore_interrupt_enable(enabled);
 	return rc;
 }
 
@@ -449,9 +457,10 @@ void
 pthread_sched_handoff(int waitstate, pthread_thread_t *pnext)
 {
 	pthread_thread_t	*pthread = CURPTHREAD();
-	int			s;
+	int			enabled;
 
-	s = splhigh();
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 
 	/*
 	 * Not sure how to handle this yet. Just yield and expect that
@@ -459,15 +468,13 @@ pthread_sched_handoff(int waitstate, pthread_thread_t *pnext)
 	 */
 	if (! waitstate) {
 		pthread_sched_wakeup(pnext, 0);
-		pthread_lock(&pthread->lock);
-		pthread_sched_dispatch(RESCHED_YIELD);		
-		splx(s);
+		pthread_sched_dispatch(RESCHED_YIELD);
+		restore_interrupt_enable(enabled);
 		return;
 	}
 
 	pthread_sched_wakeup(pnext, 0);
-	pthread_lock(&pthread->lock);
-	pthread->flags |= waitstate;
+	pthread->waitflags |= waitstate;
 	
 #ifdef  MEASURE
 	stats.handoff_attempts++;
@@ -478,8 +485,7 @@ pthread_sched_handoff(int waitstate, pthread_thread_t *pnext)
 		stats.handoff_failures++;
 #endif
 	}
-
-	splx(s);
+	restore_interrupt_enable(enabled);
 }
 
 /*
@@ -488,14 +494,14 @@ pthread_sched_handoff(int waitstate, pthread_thread_t *pnext)
 void
 pthread_sched_wakeup(pthread_thread_t *pthread, int level)
 {
-	int			s;
+	int			enabled;
 	pthread_thread_t	*pscheduler = pthread->scheduler;
 	pthread_thread_t	*inheritor, *ptmp;
 #ifdef  MEASURE
 	unsigned long		before, after;
 #endif
-	s = splhigh();
-	pthread_lock(&pthread->lock);
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 
 #ifdef  MEASURE
 	before = get_tsc();
@@ -530,7 +536,6 @@ pthread_sched_wakeup(pthread_thread_t *pthread, int level)
 			pthread->donating_to = NULL_THREADPTR;
 			pthread->wakeup_cond = WAKEUP_ALWAYS;
 		}
-		pthread_unlock(&pthread->lock);
 		goto done;
 	}
 
@@ -545,15 +550,12 @@ pthread_sched_wakeup(pthread_thread_t *pthread, int level)
 		 * to the ready state instead of donating. 
 		 */
 		inheritor = pthread->donating_to;
-		pthread_lock(&inheritor->lock);
 		
 		while (inheritor->donating_to) {
 			inheritor->schedflags = SCHED_READY;
 
 			ptmp      = inheritor;
 			inheritor = inheritor->donating_to;
-			pthread_unlock(&ptmp->lock);
-			pthread_lock(&inheritor->lock);
 		}
 		
 		/*
@@ -584,13 +586,11 @@ pthread_sched_wakeup(pthread_thread_t *pthread, int level)
 		if (inheritor != CURPTHREAD())
 			inheritor->wakeup_cond = WAKEUP_NEVER;
 
-		CURPTHREAD()->preempt = 1;
+		PREEMPT_NEEDED = CURPTHREAD()->preempt = 1;
 
 		CPUDEBUG(CORE3,
 			 "PSW1: 0x%x(%d)\n", (int) inheritor, inheritor->tid);
 			
-		pthread_unlock(&pthread->lock);
-		pthread_unlock(&inheritor->lock);
 		goto done;
 	}
 
@@ -619,14 +619,11 @@ pthread_sched_wakeup(pthread_thread_t *pthread, int level)
 		
 		pthread->schedflags = SCHED_READY;
 		inheritor = pthread->inherits_from;
-		pthread_unlock(&pthread->lock);
 		
 		pthread_sched_wakeup(inheritor, level + 1);
 	}
 	else if (pscheduler) {
 		pthread->schedflags = SCHED_READY;
-		pthread_unlock(&pthread->lock);
-		pthread_lock(&pscheduler->lock);
 		
 		/*
 		 * If the thread waking up is different than the thread
@@ -666,7 +663,6 @@ pthread_sched_wakeup(pthread_thread_t *pthread, int level)
 			pthread_sched_special_send(pscheduler, &msg);
 		}
 		else {
-			pthread_unlock(&pscheduler->lock);
 			pthread_sched_wakeup(pscheduler, level + 1);
 		}
 	}
@@ -685,7 +681,6 @@ pthread_sched_wakeup(pthread_thread_t *pthread, int level)
 			CURPTHREAD()->nextup        = pthread;
 			CURPTHREAD()->inherits_from = pthread;
 		}
-		pthread_unlock(&pthread->lock);
 	}
    done:
 #ifdef  MEASURE
@@ -698,7 +693,7 @@ pthread_sched_wakeup(pthread_thread_t *pthread, int level)
 		stats.wakeup_cycles += (after - before);
 	}
 #endif
-	splx(s);
+	restore_interrupt_enable(enabled);
 }
 
 /*
@@ -711,17 +706,17 @@ pthread_sched_thread_donate(pthread_thread_t *pchild,
 			    sched_wakecond_t wakeup_cond, oskit_s32_t timeout)
 {
 	pthread_thread_t	*pnext, *pthread = CURPTHREAD();
-	int			s;
+	int			enabled, preemptable;
 #ifdef  MEASURE
 	unsigned long		before, after;
-#endif	
-	s = splhigh();
+#endif
+	save_preemption_enable(preemptable);
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 
 #ifdef  MEASURE
 	before = get_tsc();
 #endif
-	pthread_lock(&pchild->lock);
-
 	CPUDEBUG(CORE3,
 		 "PSD0: 0x%x(%d) 0x%x(%d) %s\n",
 		 (int) pthread, pthread->tid,
@@ -737,8 +732,7 @@ pthread_sched_thread_donate(pthread_thread_t *pchild,
 			 (int) pthread, pthread->tid,
 			 (int) pchild, pchild->tid, STATESTR(pchild));
 
-		pthread_unlock(&pchild->lock);
-		splx(s);
+		restore_interrupt_enable(enabled);
 		return 0;
 	}
 
@@ -791,8 +785,6 @@ pthread_sched_thread_donate(pthread_thread_t *pchild,
 			 * unlock the current one, set the back pointer.
 			 */
 			ptmp = pnext->donating_to;
-			pthread_lock(&ptmp->lock);
-			pthread_unlock(&pnext->lock);
 			ptmp->inherits_from = pnext;
 			ptmp->nextup        = NULL_THREADPTR;
 			
@@ -823,10 +815,14 @@ pthread_sched_thread_donate(pthread_thread_t *pchild,
 		stats.donate_cycles += (after - before);
 	}
 #endif
-	thread_switch(pnext, &pthread->lock, &CURPTHREAD());
+	/* Don't worry about the lock */
+	thread_switch(pnext, &pthread->schedlock, CURPTHREAD());
+
+	/* Must set the current thread pointer! */
+	SETCURPTHREAD(pthread);
 
 	/*
-	 * Back from donation. Current thread lock is locked.
+	 * Back from donation. 
 	 *
 	 * Undo the donating link. Leave the back link alone since another
 	 * thread could have donated to the child if the chain was preempted.
@@ -835,13 +831,12 @@ pthread_sched_thread_donate(pthread_thread_t *pchild,
 	pthread->donating_to  = NULL_THREADPTR;
 	pthread->schedflags   = SCHED_RUNNING;
 
-	pthread_lock(&pchild->lock);
 	if (pchild->inherits_from == pthread)
 		pchild->inherits_from = NULL_THREADPTR;
-	pthread_unlock(&pchild->lock);
 
-	pthread_unlock(&pthread->lock);
-	splx(s);
+	restore_interrupt_enable(enabled);
+	/* Need to really *restore* the flag since we are in a new context */
+	PREEMPT_ENABLE = preemptable;
 	return 1;
 }
 
@@ -854,17 +849,17 @@ pthread_sched_thread_wait(pthread_thread_t *waiting_on,
 			  queue_head_t *queue, pthread_lock_t *plock)
 {
 	pthread_thread_t	*pthread = CURPTHREAD();
-	int			s;
+	int			enabled;
 
-	s = splhigh();
-	pthread_lock(&pthread->lock);
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 	
 	if (queue)
 		queue_enter(queue, pthread, pthread_thread_t *, chain);
 
 	/*
-	 * Safely on the queue and the current thread lock is held.
-	 * Time to unlock the given lock.
+	 * No thread locking, but need to release the provided lock
+	 * in case it was built with locks enabled.
 	 */
 	if (plock)
 		pthread_unlock(plock);
@@ -880,7 +875,7 @@ pthread_sched_thread_wait(pthread_thread_t *waiting_on,
 	if (!pthread_sched_thread_donate(waiting_on, WAKEUP_NEVER, 0))
 		pthread_sched_reschedule(RESCHED_BLOCK, &pthread->lock);
 
-	splx(s);
+	restore_interrupt_enable(enabled);
 }
 
 /*
@@ -892,7 +887,7 @@ pthread_sched_clocktick(void)
 	pthread_thread_t	*pthread;
 	int			needsoftint = 0;
 
-	if (PREEMPTBLOCKED)
+	if (! PREEMPT_ENABLE)
 		return;
 
 	pthread = CURPTHREAD();
@@ -956,8 +951,7 @@ pthread_sched_setstate(pthread_t tid, int opaque)
 	pthread_thread_t	*pthread;
 	schedmsg_t		msg;
 
-	if (tid < 0 || tid >= THREADS_MAX_THREAD ||
-	    ((pthread = tidtothread(tid)) == NULL_THREADPTR))
+	if ((pthread = tidtothread(tid)) == NULL_THREADPTR)
 		return EINVAL;
 
 	msg.type    = MSG_SCHED_SETSTATE;
@@ -970,12 +964,13 @@ pthread_sched_setstate(pthread_t tid, int opaque)
 	 */
 	pthread_sched_message_send(pthread->scheduler, &msg);	
 
-	check_yield();
 	return 0;
 }
 
 /*
- * This is the basic scheduling call invoked to donate CPU time to a thread.
+ * This is the basic scheduling call invoked to donate CPU time to a
+ * thread.  We may receive a message when some other thread wakes up
+ * from a timeout, so we disable interrupts.
  */
 int
 pthread_sched_donate_wait_recv(pthread_t tid,
@@ -983,14 +978,14 @@ pthread_sched_donate_wait_recv(pthread_t tid,
 {
 	pthread_thread_t	*pthread = CURPTHREAD();
 	pthread_thread_t	*ptarget;
-	int			rc, s;
+	int			rc;
 	
-	if (tid < 0 || tid >= THREADS_MAX_THREAD ||
-	    ((ptarget = tidtothread(tid)) == NULL_THREADPTR))
+	if ((ptarget = tidtothread(tid)) == NULL_THREADPTR)
 		panic("pthread_sched_donate: Bad TID: %d\n", tid);
 
-	s = splhigh();
-	pthread_lock(&pthread->lock);
+	assert_preemption_enabled();
+	assert_interrupts_enabled();
+	disable_interrupts();
 
 	/*
 	 * Setup the scheduler to be woken up from its donation if
@@ -1002,8 +997,7 @@ pthread_sched_donate_wait_recv(pthread_t tid,
 		 * Oops, there is a message pending so give that
 		 * back to the caller instead of doing a donation.
 		 */
-		pthread_unlock(&pthread->lock);
-		splx(s);
+		enable_interrupts();
 		return SCHEDULE_MSGRECV;
 	}
 
@@ -1019,8 +1013,7 @@ pthread_sched_donate_wait_recv(pthread_t tid,
 		 * delivered, so no need to check.
 		 */
 		pthread_sched_recv_unwait(pthread, msg);
-		pthread_unlock(&pthread->lock);
-		splx(s);
+		enable_interrupts();
 		return SCHEDULE_NOTREADY;
 	}
 
@@ -1028,14 +1021,12 @@ pthread_sched_donate_wait_recv(pthread_t tid,
 	 * Donation succeeded. There *might* be a message, so must clear
 	 * the wait and check to see if a message is pending.
 	 */
-	pthread_lock(&pthread->lock);
 	rc = pthread->donate_rc;
 
 	if (pthread_sched_recv_unwait(pthread, msg))
 		rc |= SCHEDULE_MSGRECV;
-	pthread_unlock(&pthread->lock);
 
-	splx(s);
+	enable_interrupts();
 	return rc;
 }
 
@@ -1049,15 +1040,16 @@ int
 cpuprintf(const char *fmt, ...)
 {
 	va_list	args;
-	int err, s;
+	int err, enabled;
 	char buf[256];
 
-	s = splhigh();
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 	va_start(args, fmt);
 	err = vsnprintf(buf, 256, fmt, args);
 	va_end(args);
 	write(1, buf, strlen(buf));
-	splx(s);
+	restore_interrupt_enable(enabled);
 
 	return err;
 }
@@ -1079,6 +1071,7 @@ dump_scheduler_stats()
 #endif
 #endif /* CPU_INHERIT */
 
+void
 foobar()
 {
 }
