@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -24,22 +24,25 @@ int
 pthread_detach(pthread_t tid)
 {
 	pthread_thread_t	*pthread;
-	int			p;
 
-	if (tid < 0 || tid >= THREADS_MAX_THREAD ||
-	    ((pthread = tidtothread(tid)) == NULL_THREADPTR))
+	if ((pthread = tidtothread(tid)) == NULL_THREADPTR)
 		return EINVAL;
 
-	p = splhigh();
+	assert_preemption_enabled();
+	disable_preemption();
+
 	pthread_lock(&pthread->lock);
-        if (pthread->flags & THREAD_EXITED) {
+	pthread->flags |= THREAD_DETACHED;
+	pthread_unlock(&pthread->lock);
+
+	pthread_mutex_lock(&pthread->mutex);
+	if (pthread->dead) {
+		pthread_mutex_unlock(&pthread->mutex);
 		pthread_destroy_internal(pthread);
 	}
-	else {
-		pthread->flags |= THREAD_DETACHED;
-		pthread_unlock(&pthread->lock);
-        }
-	splx(p);
+	pthread_mutex_unlock(&pthread->mutex);
+
+	enable_preemption();
 
 	return 0;
 }
