@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -21,6 +21,7 @@
  * the SCHED_FIFO and SCHED_RR policies.
  */
 #include <threads/pthread_internal.h>
+#include "pthread_signal.h"
 
 /*
  * The RunQ is a multilevel queue of doubly linked lists. Use a bitmask
@@ -37,7 +38,9 @@ static int		threads_runq_count   = 0;
 static oskit_u32_t	threads_whichrunq;
 
 /*
- * The scheduler lock is used elsewhere.
+ * The scheduler lock is used elsewhere. It must always be taken with
+ * interrupts disabled since it is used within interrupt handlers to
+ * change the run queue.
  */
 pthread_lock_t		pthread_sched_lock    = PTHREAD_LOCK_INITIALIZER;
 
@@ -189,11 +192,12 @@ int
 pthread_sched_dispatch(resched_flags_t reason, pthread_thread_t *pnext)
 {
 	pthread_thread_t	*pthread = CURPTHREAD();
-	int			s;
+	int			preemptable;
 #ifdef  MEASURE
 	unsigned long		before, after;
-#endif	
-	s = splhigh();
+#endif
+	save_preemption_enable(preemptable);
+	assert_interrupts_disabled();
 
 #ifdef  MEASURE
 	before = get_tsc();
@@ -273,7 +277,7 @@ pthread_sched_dispatch(resched_flags_t reason, pthread_thread_t *pnext)
 	/*
 	 * Clear preemption flag
 	 */
-	pthread->preempt = 0;
+	pthread->preempt = PREEMPT_NEEDED = 0;
 
 	/*
 	 * Now find a thread to run.
@@ -304,12 +308,18 @@ pthread_sched_dispatch(resched_flags_t reason, pthread_thread_t *pnext)
 		 * Avoid switch into same thread. 
 		 */
 		if (pnext == pthread) {
-			pthread_unlock(&pthread->lock);
-			splx(s);
+			pthread_unlock(&pthread->schedlock);
+
+			/*
+			 * Deliver pending signals as long as we have the
+			 * the opportunity to doso.
+			 */
+			SIGCHECK(pthread);
+			
 			return 0;
 		}
 		
-		pthread_lock(&pnext->lock);
+		pthread_lock(&pnext->schedlock);
 	}
 
 #ifdef  MEASURE
@@ -320,16 +330,22 @@ pthread_sched_dispatch(resched_flags_t reason, pthread_thread_t *pnext)
 	}
 #endif
 	/*
-	 * Switch to next thread. The thread lock for the current thread
+	 * Switch to next thread. The schedlock for the current thread
 	 * is released in the context switch code, while the lock for
 	 * next thread is carried across the context switch and is released
 	 * once the switch is committed.
 	 */
-	thread_switch(pnext, &pthread->lock, &CURPTHREAD());
+	thread_switch(pnext, &pthread->schedlock, CURPTHREAD());
+
+	/* Must set the current thread pointer! */
+	SETCURPTHREAD(pthread);
+
+	/*
+	 * Thread switch is complete. Unlock the schedlock and proceed.
+	 */
+	pthread_unlock(&pthread->schedlock);
 
 	/*
-	 * Thread has switched back in. Thread lock is locked.
-	 *
 	 * Look for exceptions before letting it return to whatever it
 	 * was doing before it yielded.
 	 */
@@ -339,10 +355,16 @@ pthread_sched_dispatch(resched_flags_t reason, pthread_thread_t *pnext)
 		 * be allowed to return, even if it was killed, so the driver
 		 * state can be cleaned up. The death will be noticed later.
 		 */
-		;
+		goto done;
 	}
-	else if ((pthread->flags &
-		  (THREAD_KILLED|THREAD_EXITING)) == THREAD_KILLED) {
+
+	/*
+	 * These flag bits are protected by the pthread lock.
+	 */
+	pthread_lock(&pthread->lock);
+
+	if ((pthread->flags &
+	     (THREAD_KILLED|THREAD_EXITING)) == THREAD_KILLED) {
 		/*
 		 * Thread was canceled. Time to actually reap it, but only
 		 * if its not already in the process of exiting.
@@ -360,10 +382,26 @@ pthread_sched_dispatch(resched_flags_t reason, pthread_thread_t *pnext)
 			 * Never returns
 			 */
 		}
+		else {
+			pthread_unlock(&pthread->lock);
+			goto done;
+		}
 	}
-
 	pthread_unlock(&pthread->lock);
-	splx(s);
+
+	/*
+	 * Look for signals. Deliver them in the context of the thread.
+	 */
+	SIGCHECK(pthread);
+
+	/* Need to really *restore* the flag since we are in a new context */
+	PREEMPT_ENABLE = preemptable;
+	return 1;
+	
+
+   done:
+	/* Need to really *restore* the flag since we are in a new context */
+	PREEMPT_ENABLE = preemptable;
 	return 1;
 }
 
@@ -378,26 +416,28 @@ int
 pthread_sched_reschedule(resched_flags_t reason, pthread_lock_t *plock)
 {
 	pthread_thread_t	*pthread = CURPTHREAD();
-	int			s, rc;
+	int			enabled, rc;
 
-	s = splhigh();
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 
 	/*
-	 * Need to lock the thread lock and then release the provided
-	 * spinlock. This provides atomicity with regards to the thread
-	 * that is switching out since in all cases the lock is either
-	 * the thread lock, or a queue that it just put itself on.
+	 * Lock the thread schedlock. This provides atomicity with respect
+	 * to the switch, since another CPU will will not be able to switch
+	 * into a thread until it is completely switched out someplace else.
+	 * Once the schedlock is taken, the provided lock can be released,
+	 * since the thread can now be woken up, although it cannot be
+	 * switched into for the reason just given.
 	 */
-	if (plock != &(pthread->lock)) {
-		pthread_lock(&(pthread->lock));
-		
-		if (plock)
-			pthread_unlock(plock);
-	}
+	assert(plock != &(pthread->lock));
+
+	pthread_lock(&(pthread->schedlock));
+	if (plock)
+		pthread_unlock(plock);
 
 	rc = pthread_sched_dispatch(reason, 0);
 
-	splx(s);
+	restore_interrupt_enable(enabled);
 	return rc;
 }
 
@@ -408,70 +448,48 @@ void
 pthread_sched_handoff(int waitstate, pthread_thread_t *pnext)
 {
 	pthread_thread_t	*pthread = CURPTHREAD();
-	int			s;
+	int			enabled;
+
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 
-	s = splhigh();
-	pthread_lock(&pnext->lock);
-	pthread_lock(&pthread->lock);
-	
 	if (waitstate) {
-		pthread->flags |= waitstate;
+		pthread_lock(&pthread->waitlock);
+		pthread->waitflags |= waitstate;
+		pthread_lock(&pthread->schedlock);
+		pthread_lock(&pnext->schedlock);
+		pthread_unlock(&pthread->waitlock);
 		pthread_sched_dispatch(RESCHED_BLOCK, pnext);
 	}
-	else
+	else {
+		pthread_lock(&pthread->schedlock);
+		pthread_lock(&pnext->schedlock);
 		pthread_sched_dispatch(RESCHED_YIELD, pnext);
-	
-	splx(s);
-}
-
-/*
- * Wakeup a thread. This does not schedule the thread, but only terminates
- * the wait state. If the thread was canceled, tell the caller.
- */
-int
-pthread_sched_thread_wakeup(int waitstate, pthread_thread_t *pthread)
-{
-	int		s, okay = 1;
-
-	s = splhigh();
-	pthread_lock(&pthread->lock);
-	pthread->flags &= ~waitstate;
-
-	if (pthread->flags & THREAD_CANCELED) {
-		pthread_unlock(&pthread->lock);
-		pthread_sched_setrunnable(pthread);
-		okay = 0;
 	}
-	else
-		pthread_unlock(&pthread->lock);
 	
-	splx(s);
-	return okay;
+	restore_interrupt_enable(enabled);
 }
 
 /*
- * Place a thread that was blocked for some reason, back on the runq.
- * Both the thread lock and the scheduler lock is taken. That is not
- * strictly necessary since there is only a single scheduler, even in
- * SMP mode, but it will indicate where deadlock might happen.
+ * Place a thread that was blocked, back on the runq. The pthread lock
+ * is *not* taken. This routine is called from interrupt level to
+ * reschedule threads waiting in timed conditions or sleep.
  *
  * Return a boolean value indicating whether the current thread is still
- * the thread that should be running. In this case, it has to still be
- * the highest priority thread. The caller will need to take appropriate
- * action, if any.
+ * the thread that should be running.
  */
 int 
 pthread_sched_setrunnable(pthread_thread_t *pthread)
 {
-	int		s;
+	int		enabled, resched = 0;
 #ifdef  MEASURE
 	unsigned long	before, after;
-#endif	
-	s = splhigh();
+#endif
+	save_interrupt_enable(enabled);
+	disable_interrupts();
 #ifdef  MEASURE
 	before = get_tsc();
 #endif
-	pthread_lock(&pthread->lock);
 	pthread_lock(&pthread_sched_lock);
 	
 	if (pthread_runq_onrunq(pthread))
@@ -481,9 +499,8 @@ pthread_sched_setrunnable(pthread_thread_t *pthread)
 	pthread_runq_insert_tail(pthread);
 
 	if (CURPTHREAD()->priority < pthread_runq_maxprio())
-		CURPTHREAD()->preempt = 1;
+		resched = PREEMPT_NEEDED = 1;
 
-	pthread_unlock(&pthread->lock);
 	pthread_unlock(&pthread_sched_lock);
 #ifdef  MEASURE
 	after = get_tsc();
@@ -492,9 +509,9 @@ pthread_sched_setrunnable(pthread_thread_t *pthread)
 		stats.wakeup_cycles += (after - before);
 	}
 #endif
-	splx(s);
+	restore_interrupt_enable(enabled);
 
-	return CURPTHREAD()->preempt;
+	return resched;
 }
 
 /*
@@ -503,13 +520,16 @@ pthread_sched_setrunnable(pthread_thread_t *pthread)
  * caller, if the change requires a reschedule.
  *
  * Currently, the priority inheritance code is not SMP safe. Also, it is
- * specific to this scheduler module.  */
+ * specific to this scheduler module.
+ */
 int
 pthread_sched_change_state(pthread_thread_t *pthread, int newprio, int policy)
 {
-	int	s;
+	int		resched = 0;
+	
+	assert_interrupts_enabled();
+	disable_interrupts();
 
-	s = splhigh();
 	pthread_lock(&pthread->lock);
 
 	switch (policy) {
@@ -570,16 +590,230 @@ pthread_sched_change_state(pthread_thread_t *pthread, int newprio, int policy)
 	}
 #endif	
 	if (CURPTHREAD()->priority < pthread_runq_maxprio())
-		CURPTHREAD()->preempt = 1;
+		resched = PREEMPT_NEEDED = 1;
 	
 	pthread_unlock(&pthread_sched_lock);
    done:
 	pthread_unlock(&pthread->lock);
-	splx(s);
+	enable_interrupts();
 
-	return CURPTHREAD()->preempt;
+	return resched;
 }
 
+#ifdef  PRI_INHERIT
+#ifdef  SMP
+Sorry, this code does not work in SMP mode. No locking at all!
+#endif
+
+/*
+ * This priority inheritance algorithm is based on the paper "Removing
+ * Priority Inversion from an Operating System," by Steven Sommer.
+ */
+int	threads_priority_debug = 0;
+
+void	pthread_priority_decreasing_recompute(pthread_thread_t *pthread);
+void	pthread_priority_increasing_recompute(pthread_thread_t *pthread);
+
+OSKIT_INLINE void
+threads_change_priority(pthread_thread_t *pthread, int newprio)
+{
+	if (pthread_runq_onrunq(pthread)) {
+		pthread_runq_remove(pthread);
+		pthread->priority = newprio;
+		pthread_runq_insert_tail(pthread);
+	}
+	else
+		pthread->priority = newprio;
+}
+
+void
+pthread_priority_inherit(pthread_thread_t *pwaiting_for)
+{
+	pthread_thread_t	*pthread = CURPTHREAD();
+	int			enabled;
+
+	save_interrupt_enable(enabled);
+	disable_interrupts();
+
+	/*
+	 * Add blocked thread to targets' list of waiters.
+	 */
+	queue_enter(&pwaiting_for->waiters, pthread,
+		    pthread_thread_t *, waiters_chain);
+
+	/*
+	 * Set the waiting link so that the chain of waiters
+	 * can be followed when doing the priority transfer.
+	 */
+	pthread->waiting_for = pwaiting_for;
+
+	pthread_priority_increasing_recompute(pthread);
+
+	restore_interrupt_enable(enabled);
+}
+
+/*
+ * Do a transitive priority transfer. Following the waiting_for links,
+ * transfering higher priority to lower priority threads.
+ *
+ * pthread is locked.
+ */
+void
+pthread_priority_increasing_recompute(pthread_thread_t *pthread)
+{
+	pthread_thread_t	*pwaiting_for = pthread->waiting_for;
+
+	do {
+		if (pthread->priority <= pwaiting_for->priority)
+			break;
+
+		if (threads_priority_debug)
+			printf("Inherit: Transferring priority: "
+			       "From %p(%d/%d) to %p(%d/%d)\n",
+			       pthread, (int) pthread->tid, pthread->priority,
+			       pwaiting_for, (int) pwaiting_for->tid,
+			       pwaiting_for->priority);
+
+		threads_change_priority(pwaiting_for, pthread->priority);
+		pwaiting_for->inherits_from = pthread;
+		
+		pthread = pwaiting_for;
+		pwaiting_for = pwaiting_for->waiting_for;
+	} while (pwaiting_for);
+}	
+
+/*
+ * Undo the effects of a priority inheritance after a thread unblocks
+ * another thread.
+ */
+void
+pthread_priority_uninherit(pthread_thread_t *punblocked)
+{
+	pthread_thread_t	*pthread = CURPTHREAD();
+	pthread_thread_t	*ptmp1;
+	int			enabled;
+
+	save_interrupt_enable(enabled);
+	disable_interrupts();
+	
+	/*
+	 * Remove the unblocked thread from the set of threads blocked
+	 * by the current thread.
+	 */
+	queue_remove(&pthread->waiters,
+		     punblocked, pthread_thread_t *, waiters_chain);
+
+	/*
+	 * The rest of the waiters are now waiting on the unblocked thread
+	 * since it won the prize. Indicate that, and add the entire set
+	 * of threads to the list of threads that punblocked is blocking.
+	 */
+	while (! queue_empty(&pthread->waiters)) {
+		queue_remove_first(&pthread->waiters,
+				   ptmp1, pthread_thread_t *, waiters_chain);
+
+		ptmp1->waiting_for = punblocked;
+		
+		queue_enter(&punblocked->waiters,
+			    ptmp1, pthread_thread_t *, waiters_chain);
+	}
+	punblocked->waiting_for = NULL_THREADPTR;
+
+	/*
+	 * Now recompute the priorities.
+	 */
+	if (pthread->inherits_from)
+		pthread_priority_decreasing_recompute(pthread);
+		      
+	restore_interrupt_enable(enabled);
+}
+
+void
+pthread_priority_decreasing_recompute(pthread_thread_t *pthread)
+{
+	int			priority = pthread->priority;
+	pthread_thread_t	*pnext = 0, *ptmp;
+	int			maxpri = -1;
+
+	/*
+	 * Find the highest priority thread from the set of threads
+	 * waiting on pthread.
+	 */
+	queue_iterate(&pthread->waiters, ptmp, pthread_thread_t *, chain) {
+		if (ptmp->priority > maxpri) {
+			maxpri = ptmp->priority;
+			pnext  = ptmp;
+		}
+	}
+
+	/*
+	 * If there is a waiting thread, and its priority is greater
+	 * then this threads' priority, then inherit priority
+	 * from that thread. Otherwise, reset this threads' priority
+	 * back to its base priority since either there is nothing to
+	 * inherit from (empty waiters), or its base priority is better
+	 * than any of the waiting threads.
+	 */
+	if (pnext) {
+		if (threads_priority_debug)
+			printf("Uninherit: Transferring priority: "
+			       "From %p(%d/%d) to %p(%d/%d)\n",
+			       pnext, (int) pnext->tid, pnext->priority,
+			       pthread, (int) pthread->tid, pthread->priority);
+
+		if (pnext->priority > pthread->base_priority) {
+			threads_change_priority(pthread, pnext->priority);
+			pthread->inherits_from = pnext;
+		}
+	}
+	else {
+		if (threads_priority_debug)
+			printf("Resetting priority back to base: "
+			       "%p(%d) - %d to %d\n",
+			       pthread, (int) pthread->tid,
+			       pthread->priority, pthread->base_priority);
+
+		threads_change_priority(pthread, pthread->base_priority);
+		pthread->inherits_from = 0;
+	}
+
+	/*
+	 * If this threads' priority was lowered, and another thread is
+	 * inheriting from it, must propogate the lowered priority down
+	 * the chain. This would not happen when a thread unblocks another
+	 * thread. It could happen if an external event caused a blocked
+	 * thread to change state, and that thread was donating its priority.
+	 */
+	if (pthread->priority < priority) {
+		if (pthread->waiting_for &&  
+		    pthread->waiting_for->inherits_from == pthread) {
+			pthread_priority_decreasing_recompute(pthread->
+							      waiting_for);
+		}
+	}
+}
+
+/*
+ * A thread is killed, but waiting on a thread. Must undo the inheritance.
+ *
+ * Interrupts should be disabled, and the thread locked.
+ */
+void
+pthread_priority_kill(pthread_thread_t *pthread)
+{
+	pthread_thread_t	*pwaiting_for = pthread->waiting_for;
+	
+	queue_remove(&pwaiting_for->waiters, pthread,
+		     pthread_thread_t *, waiters_chain);
+
+	pthread->waiting_for = NULL_THREADPTR;
+	
+	if (pwaiting_for->inherits_from == pthread)
+		pthread_priority_decreasing_recompute(pwaiting_for);
+}
+#endif
+
+
 #ifdef	MEASURE
 void
 dump_scheduler_stats()
