@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -21,19 +21,19 @@
 #include <threads/pthread_internal.h>
 
 int
-pthread_setprio(pthread_t tid, int prio)
+oskit_pthread_setprio(pthread_t tid, int prio)
 {
 	pthread_thread_t  *pthread;
 
-	if (tid < 0 || tid >= THREADS_MAX_THREAD ||
-	    tidtothread(tid) == NULL_THREADPTR)
+	if ((pthread = tidtothread(tid)) == NULL_THREADPTR)
 		return EINVAL;
 
-	pthread = tidtothread(tid);
-
 	if (prio < PRIORITY_MIN || prio > PRIORITY_MAX)
 		return EINVAL;
 
+	assert_preemption_enabled();
+	disable_preemption();
+
 	/*
 	 * Pass this off to the scheduler module. The simple POSIX
 	 * model allows the policy and priority to be changed, so
@@ -41,11 +41,6 @@ pthread_setprio(pthread_t tid, int prio)
 	 */
 	pthread_sched_change_state(pthread, prio, -1);
 
-	/*
-	 * Check for reschedule since current thread might no longer be the
-	 * highest priority thread.
-	 */
-	check_yield();
-	
+	enable_preemption();
 	return 0;
 }
