@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1996, 1998, 1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -26,38 +26,19 @@ pthread_setschedparam(pthread_t tid, int policy,
 {
 	pthread_thread_t  *pthread;
 
-	if (tid < 0 || tid >= THREADS_MAX_THREAD ||
-	    tidtothread(tid) == NULL_THREADPTR)
+	if ((pthread = tidtothread(tid)) == NULL_THREADPTR)
 		return EINVAL;
 
-	pthread = tidtothread(tid);
+	assert_preemption_enabled();
+	disable_preemption();
 
-#ifndef CPU_INHERIT
-	switch (policy) {
-	case SCHED_FIFO:
-	case SCHED_RR:
-		break;
-
-	case SCHED_DECAY:
-		panic("pthread_setschedparam: SCHED_DECAY not supported yet");
-		break;
-
-	default:
-		return EINVAL;
-	}
-#endif
 	/*
 	 * Pass this off to the scheduler module. The simple POSIX
 	 * model allows the policy and priority to be changed.
 	 */
 	pthread_sched_change_state(pthread, param->priority, policy);
 
-	/*
-	 * Check for reschedule since current thread might no longer be the
-	 * highest priority thread.
-	 */
-	check_yield();
-	
+	enable_preemption();
 	return 0;
 }
 
