@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-2000 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -20,7 +20,7 @@
  *
  * We only implement the parts that make sense.
  *
- * XXXXXX: The problem spots are stat and getdirents. 
+ * XXXXXX: The problem spots are stat and getdirents.
  */
 #include <oskit/fs/dir.h>
 #include <oskit/fs/openfile.h>
@@ -30,16 +30,6 @@
 #include <oskit/time.h>
 #include <oskit/queue.h>
 
-#include <sys/types.h>
-#include <string.h>
-#include <assert.h>
-#include <errno.h>
-#include <stdarg.h>
-#include <malloc.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <stddef.h>
-
 #include "native.h"
 #include "support.h"
 
@@ -47,63 +37,9 @@
 #define VERBOSITY 	0
 #endif
 
-oskit_u64_t	NATIVEOS(lseek)(int, oskit_u64_t, int);
-int		NATIVEOS(truncate)(char *, int, oskit_u64_t);
-
-/*
- * These are some fcntl.h flags that are different in the Oskit headers
- */
-#define	O_RDONLY	0x0000		/* open for reading only */
-#define	O_WRONLY	0x0001		/* open for writing only */
-#define	O_RDWR		0x0002		/* open for reading and writing */
-#define	O_APPEND	0x0008		/* set append mode */
-#define	O_CREAT		0x0200		/* create if nonexistent */
-#define	O_TRUNC		0x0400		/* truncate to zero length */
-#define	O_EXCL		0x0800		/* error if already exists */
-
-/*
- * oskit header defs are different than FreeBSD. Looks like we can include
- * this directly since we don't use any of the structures.
- */
-#include "freebsd/src/sys/sys/fcntl.h"
-
-/*
- * The oskit version of the stat buffer is different than the native
- * FreeBSD version.
- */
-struct bsdstat {
-	oskit_u32_t	st_dev;		/* inode's device */
-	oskit_u32_t	st_ino;		/* inode's number */
-	oskit_u16_t	st_mode;	/* inode protection mode */
-	oskit_u16_t	st_nlink;	/* number of hard links */
-	oskit_u32_t	st_uid;		/* user ID of the file's owner */
-	oskit_u32_t	st_gid;		/* group ID of the file's group */
-	oskit_u32_t	st_rdev;	/* device type */
-	oskit_timespec_t st_atimespec;	/* time of last access */
-	oskit_timespec_t st_mtimespec;	/* time of last data modification */
-	oskit_timespec_t st_ctimespec;	/* time of last file status change */
-	oskit_s64_t	st_size;	/* file size, in bytes */
-	oskit_s64_t	st_blocks;	/* blocks allocated for file */
-	oskit_u32_t	st_blksize;	/* optimal blocksize for I/O */
-	oskit_u32_t	st_flags;	/* user defined flags for file */
-	oskit_u32_t	st_gen;		/* file generation number */
-	oskit_u32_t	st_lspare;
-	oskit_s64_t	st_qspare[2];
-};
-#define	ALLPERMS	(OSKIT_S_ISUID|OSKIT_S_ISGID|OSKIT_S_ISVTX|\
+#define	OSKIT_ALLPERMS	(OSKIT_S_ISUID|OSKIT_S_ISGID|OSKIT_S_ISVTX|\
 			 OSKIT_S_IRWXU|OSKIT_S_IRWXG|OSKIT_S_IRWXO)
 
-/*
- * Same for dirents. The oskit version is quite a bit simpler.
- */
-struct bsddirent {
-	oskit_u32_t	d_fileno;	/* file number of entry */
-	oskit_u16_t	d_reclen;	/* length of this record */
-	oskit_u8_t	d_type; 	/* file type, see below */
-	oskit_u8_t	d_namlen;	/* length of string in d_name */
-	char		d_name[256];	/* name must be no longer than this */
-};
-
 /*
  * structures to describe a native file or directory. Note how similar
  * they are. As it should (better) be.
@@ -117,6 +53,8 @@ struct fs_file {
 	char		*component;	/* Component name */
 	int		complen;	/* Length of component name */
 	queue_chain_t	chain;		/* siblings in contents chain */
+	oskit_mode_t	mode;		/* Mode for creation */
+	char		create;		/* Needs an actual disk file created */
 };
 
 struct fs_dir {
@@ -157,6 +95,9 @@ static struct oskit_file_ops		fs_file_ops;
 static struct oskit_dir_ops		fs_dir_ops;
 static struct oskit_openfile_ops	fs_openfile_ops;
 static struct fs_dir			*fs_dir_rootdir;
+static oskit_error_t		fs_dirents_create(oskit_dirent_t *entries,
+					int count,
+					oskit_dirents_t **out_dirents);
 
 /*
  * Create dir/file impls.
@@ -169,12 +110,12 @@ fs_file_allocate(char *pathname, const char *component)
 
         if (newfsf == NULL)
                 return 0;
-	
+
         memset(newfsf, 0, sizeof(*newfsf));
 
 	complen = strlen(component);
 	pathlen = strlen(pathname);
-	
+
 	if ((newfsf->pathname = malloc(complen + pathlen + 1)) == NULL) {
 		free(newfsf);
 		return 0;
@@ -192,19 +133,19 @@ fs_file_allocate(char *pathname, const char *component)
 }
 
 static struct fs_dir *
-fs_dir_allocate(char *pathname, const char *component)
+fs_dir_allocate(char *pathname, const char *component, oskit_dir_t *parent)
 {
         struct fs_dir	*newfsd = malloc(sizeof(*newfsd));
 	int		pathlen, complen;
 
         if (newfsd == NULL)
                 return 0;
-	
+
         memset(newfsd, 0, sizeof(*newfsd));
 
 	complen = strlen(component);
 	pathlen = strlen(pathname);
-	
+
 	if ((newfsd->pathname = malloc(complen + pathlen + 2)) == NULL) {
 		free(newfsd);
 		return 0;
@@ -220,6 +161,11 @@ fs_dir_allocate(char *pathname, const char *component)
 	newfsd->fd         = -1;
 	queue_init(&newfsd->contents);
 
+	if (parent) {
+		newfsd->parent = parent;
+		oskit_dir_addref(parent);
+	}
+
 	return newfsd;
 }
 
@@ -230,14 +176,14 @@ fs_openfile_allocate(struct fs_file *fsf, int fd)
 
         if (ofile == NULL)
                 return 0;
-	
+
         memset(ofile, 0, sizeof(*ofile));
 
         ofile->count      = 1;
         ofile->ofilei.ops = &fs_openfile_ops;
 	ofile->fd         = fd;
 	ofile->fsf        = fsf;
-	oskit_file_addref(&fsf->filei);	
+	oskit_file_addref(&fsf->filei);
 
 	return ofile;
 }
@@ -292,7 +238,7 @@ static OSKIT_COMDECL_U
 fs_file_addref(oskit_file_t *f)
 {
         struct fs_file *fsf = (struct fs_file *) f;
-	
+
 	assert(fsf->count);
 	return ++fsf->count;
 }
@@ -306,8 +252,25 @@ fs_file_release(oskit_file_t *f)
 
 	if (--fsf->count)
 		return fsf->count;
-	free(fsf->pathname);
 
+	if (fsf->fd >= 0)
+		NATIVEOS(close)(fsf->fd);
+
+	/*
+	 * If the create flag is set, this was a delayed directory entry
+	 * creation. Go ahead and create the file on disk now, with the
+	 * mode that was specified when fs_dir_create() was called.
+	 */
+	if (fsf->create) {
+		int fd, flags = O_WRONLY|O_CREAT|O_EXCL;
+
+		if ((fd = NATIVEOS(open)(fsf->pathname, flags, fsf->mode)) < 0)
+			panic("fs_file_release: File should not exist!");
+
+		NATIVEOS(close)(fd);
+	}
+
+	free(fsf->pathname);
 	free(fsf);
 	return 0;
 }
@@ -338,14 +301,14 @@ static OSKIT_COMDECL
 fs_file_stat(oskit_file_t *f, struct oskit_stat *out_stats)
 {
         struct fs_file *fsf = (struct fs_file *) f;
-	struct bsdstat sb;
+	struct stat sb;
 
 #if VERBOSITY > 1
 	printf(__FUNCTION__": asked to stat `%s'\n", fsf->pathname);
 #endif
 
-	if (NATIVEOS(stat)(fsf->pathname, &sb) < 0)
-		return errno;
+	if (NATIVEOS(lstat)(fsf->pathname, &sb) < 0)
+		return native_to_oskit_error(native_errno);
 
 #if VERBOSITY > 1
 	printf(__FUNCTION__": stat completed `%s'\n", fsf->pathname);
@@ -357,12 +320,12 @@ fs_file_stat(oskit_file_t *f, struct oskit_stat *out_stats)
 	out_stats->uid = sb.st_uid;
 	out_stats->gid = sb.st_gid;
 	out_stats->rdev = sb.st_rdev;
-	out_stats->atime.tv_sec = sb.st_atimespec.tv_sec;
-	out_stats->atime.tv_nsec = sb.st_atimespec.tv_nsec;
-	out_stats->mtime.tv_sec = sb.st_mtimespec.tv_sec;
-	out_stats->mtime.tv_nsec = sb.st_mtimespec.tv_nsec;
-	out_stats->ctime.tv_sec = sb.st_ctimespec.tv_sec;
-	out_stats->ctime.tv_nsec = sb.st_ctimespec.tv_nsec;
+	out_stats->atime.tv_sec = STATATIMESEC(sb);
+	out_stats->atime.tv_nsec = STATATIMENSEC(sb);
+	out_stats->mtime.tv_sec = STATMTIMESEC(sb);
+	out_stats->mtime.tv_nsec = STATMTIMENSEC(sb);
+	out_stats->ctime.tv_sec = STATCTIMESEC(sb);
+	out_stats->ctime.tv_nsec = STATCTIMENSEC(sb);
 	out_stats->size = sb.st_size;
 	out_stats->blocks = sb.st_blocks;
 	out_stats->blksize = sb.st_blksize;
@@ -375,14 +338,14 @@ fs_file_setstat(oskit_file_t *f,
 		oskit_u32_t mask, const struct oskit_stat *stats)
 {
         struct fs_file  *fsf = (struct fs_file *) f;
-	struct bsdstat  sb;
+	struct stat sb;
 
 #if VERBOSITY > 1
 	printf(__FUNCTION__": asked to setstat `%s'\n", fsf->pathname);
 #endif
 
-	if (NATIVEOS(stat)(fsf->pathname, &sb) < 0)
-		return errno;
+	if (NATIVEOS(lstat)(fsf->pathname, &sb) < 0)
+		return native_to_oskit_error(native_errno);
 
 	/*
 	 * These operations cannot be applied to a symbolic link
@@ -395,26 +358,48 @@ fs_file_setstat(oskit_file_t *f,
 	 * Only allow a few things right now.
 	 */
 	if (mask & ~(OSKIT_STAT_MODE|OSKIT_STAT_UID|
-		     OSKIT_STAT_SIZE|OSKIT_STAT_GID))
+		     OSKIT_STAT_SIZE|OSKIT_STAT_GID|
+		     OSKIT_STAT_ATIME|OSKIT_STAT_MTIME))
 		return OSKIT_E_INVALIDARG;
 
 	if (mask & OSKIT_STAT_MODE) {
-		if (NATIVEOS(chmod)(fsf->pathname, stats->mode & ALLPERMS) < 0)
-			return errno;
+		if (NATIVEOS(chmod)(fsf->pathname, /* 07777 are portable */
+				    stats->mode & OSKIT_ALLPERMS) < 0)
+			return native_to_oskit_error(native_errno);
 	}
 	if (mask & OSKIT_STAT_UID) {
 		if (NATIVEOS(chown)(fsf->pathname, stats->uid, -1) < 0)
-			return errno;
+			return native_to_oskit_error(native_errno);
 	}
 	if (mask & OSKIT_STAT_GID) {
 		if (NATIVEOS(chown)(fsf->pathname, -1, stats->gid) < 0)
-			return errno;
+			return native_to_oskit_error(native_errno);
 	}
 	if (mask & OSKIT_STAT_SIZE) {
-		if (NATIVEOS(truncate)(fsf->pathname, 0, stats->size) < 0)
-			return errno;
+		if (NATIVEOS(truncate)(fsf->pathname, stats->size) < 0)
+			return native_to_oskit_error(native_errno);
 	}
-	
+	if (mask & (OSKIT_STAT_ATIME|OSKIT_STAT_MTIME)) {
+		struct timeval timevals[2];
+
+		if (mask & OSKIT_STAT_ATIME) {
+			TIMESPEC_TO_TIMEVAL(&timevals[0], &stats->atime);
+		}
+		else {
+			NATIVEOS(gettimeofday)(&timevals[0], NULL);
+		}
+
+		if (mask & OSKIT_STAT_MTIME) {
+			TIMESPEC_TO_TIMEVAL(&timevals[1], &stats->mtime);
+		}
+		else {
+			NATIVEOS(gettimeofday)(&timevals[1], NULL);
+		}
+
+		if (NATIVEOS(utimes)(fsf->pathname, timevals) < 0)
+			return native_to_oskit_error(native_errno);
+	}
+
 	return 0;
 }
 
@@ -432,7 +417,7 @@ fs_file_sync(oskit_file_t *f, oskit_bool_t wait)
 	assert(fsf->fd);
 
 	if (NATIVEOS(fsync)(fsf->fd) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	return 0;
 }
@@ -446,7 +431,7 @@ fs_file_access(oskit_file_t *f, oskit_amode_t mask)
 	printf(__FUNCTION__": asked to access on `%s'\n", fsf->pathname);
 #endif
 	if (NATIVEOS(access)(fsf->pathname, mask) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	return 0;
 }
@@ -459,7 +444,7 @@ fs_file_readlink(oskit_file_t *f, char *buf, oskit_u32_t len,
 	int		count;
 
 	if ((count = NATIVEOS(readlink)(fsf->pathname, buf, len)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	*out_actual = count;
 
@@ -472,7 +457,7 @@ fs_file_open(oskit_file_t *f, oskit_oflags_t iflags,
 {
         struct fs_file  *fsf = (struct fs_file *) f;
 	struct fs_open  *fso;
-	int		flags, fd;
+	int		mode = 0, flags = 0, fd;
 
 #if VERBOSITY > 1
 	printf(__FUNCTION__": asked to open `%s'\n", fsf->pathname);
@@ -484,7 +469,7 @@ fs_file_open(oskit_file_t *f, oskit_oflags_t iflags,
 		flags = O_WRONLY;
 	if ((iflags & OSKIT_O_ACCMODE) == OSKIT_O_RDWR)
 		flags = O_RDWR;
-	
+
 	if (iflags & OSKIT_O_TRUNC)
 		flags |= O_TRUNC;
 	if (iflags & OSKIT_O_APPEND)
@@ -492,11 +477,23 @@ fs_file_open(oskit_file_t *f, oskit_oflags_t iflags,
 	if (iflags & OSKIT_O_CREAT)
 		flags |= O_CREAT;
 
-	if ((fd = NATIVEOS(open)(fsf->pathname, flags, 0)) < 0)
-		return errno;
+	/*
+	 * If the create flag is set, this was a delayed directory entry
+	 * creation. Use the mode bits that were specified when
+	 * fs_dir_create() was called so that file is created with the
+	 * proper bits. Add in O_EXCL too just to be safe.
+	 */
+	if (fsf->create) {
+		flags |= O_CREAT|O_EXCL;
+		mode   = fsf->mode;
+		fsf->create = 0;
+	}
+
+	if ((fd = NATIVEOS(open)(fsf->pathname, flags, mode)) < 0)
+		return native_to_oskit_error(native_errno);
 
 	/*
-	 * Open succeeded, so create an openfile for it. 
+	 * Open succeeded, so create an openfile for it.
 	 */
 	if ((fso = fs_openfile_allocate(fsf, fd)) == NULL) {
 		NATIVEOS(close)(fd);
@@ -522,7 +519,7 @@ fs_dir_open(oskit_dir_t *d, oskit_oflags_t iflags,
 		flags = O_WRONLY;
 	if ((iflags & OSKIT_O_ACCMODE) == OSKIT_O_RDWR)
 		flags = O_RDWR;
-	
+
 	if (iflags & OSKIT_O_TRUNC)
 		flags |= O_TRUNC;
 	if (iflags & OSKIT_O_APPEND)
@@ -531,10 +528,10 @@ fs_dir_open(oskit_dir_t *d, oskit_oflags_t iflags,
 		flags |= O_CREAT;
 
 	if ((fd = NATIVEOS(open)(fsd->pathname, flags, 0)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	/*
-	 * Open succeeded, so create an openfile for it. 
+	 * Open succeeded, so create an openfile for it.
 	 */
 	if ((fso = fs_openfile_allocate((struct fs_file *)fsd, fd)) == NULL) {
 		NATIVEOS(close)(fd);
@@ -556,9 +553,9 @@ fs_lookup(struct fs_dir *fsd, const char *name)
 
 	if (len == 0 || (len == 1 && name[0] == '.'))
 		return (oskit_file_t *) &fsd->diri;
-	
+
 	if (len == 2 && name[0] == '.' && name[1] == '.')
-		return (oskit_file_t *) fsd->parent ?: &fsd->diri;
+		return (oskit_file_t *) (fsd->parent ?: &fsd->diri);
 
 	queue_iterate(&fsd->contents, fsf, struct fs_file *, chain) {
 		if (fsf->complen == len &&
@@ -593,7 +590,7 @@ fs_dir_lookup(oskit_dir_t *d, const char *name, oskit_file_t **out_file)
         struct fs_dir	*fsd = (struct fs_dir *) d;
 	struct fs_file  *fsf;
 	oskit_file_t	*file;
-	struct bsdstat  sb;
+	struct stat	sb;
 	char		fname[MAXPATHLEN];
 
 #if VERBOSITY > 1
@@ -619,8 +616,8 @@ fs_dir_lookup(oskit_dir_t *d, const char *name, oskit_file_t **out_file)
 	 */
 	strcpy(fname, fsd->pathname);
 	strcat(fname, name);
-	if (NATIVEOS(stat)(fname, &sb) < 0)
-		return errno;
+	if (NATIVEOS(lstat)(fname, &sb) < 0)
+		return native_to_oskit_error(native_errno);
 
 #if VERBOSITY > 1
 	printf(__FUNCTION__": On disk: `%s'\n", fname);
@@ -630,13 +627,14 @@ fs_dir_lookup(oskit_dir_t *d, const char *name, oskit_file_t **out_file)
 	 * It does. Create a fs_file or fs_dir entry and link it in.
 	 */
 	if (sb.st_mode & OSKIT_S_IFDIR)
-		fsf = (struct fs_file *) fs_dir_allocate(fsd->pathname, name);
+		fsf = (struct fs_file *)
+			fs_dir_allocate(fsd->pathname, name, d);
 	else
 		fsf = fs_file_allocate(fsd->pathname, name);
-	
+
 	if (fsf == NULL)
 		return OSKIT_ENOMEM;
-	
+
 	fs_contents_link(fsd, (struct fs_dir *) fsf);
 
 	/*
@@ -666,38 +664,73 @@ fs_dir_create(oskit_dir_t *d, const char *name,
 
 		fsf = fs_file_allocate(fsd->pathname, name);
 
-		if (!fsf)
+		if (!fsf) {
 			rc = OSKIT_ENOMEM;
-		else {
-			int fd, flags = O_WRONLY|O_CREAT|O_TRUNC;
-			
-			/*
-			 * Now create the actual file on disk. 
-			 */
-			if ((fd =
-			     NATIVEOS(open)(fsf->pathname, flags, mode)) < 0) {
-				oskit_file_release(&fsf->filei);
-				return errno;
-			}
-			NATIVEOS(close)(fd);
-			
-			fs_contents_link(fsd, (struct fs_dir *) fsf);
+			goto done;
+		}
 
+		fs_contents_link(fsd, (struct fs_dir *) fsf);
+
+		/*
+		 * See if the file already exists on disk. With excl set
+		 * it must not exist.
+		 */
+		if (NATIVEOS(access)(fsf->pathname, 0) == 0) {
 			/*
+			 * It exists. We created a directory entry, but
+			 * that is okay. Return error.
+			 */
+			if (excl) {
+				rc = OSKIT_EEXIST;
+				goto done;
+			}
+			/*
+			 * Otherwise, return a handle to the directory entry.
+			 *
 			 * The directory holds one ref for the file object,
 			 * and we add another for the pointer going out.
 			 */
 			++fsf->count;
 			*out_file = &fsf->filei;
 			rc = 0;
+			goto done;
 		}
-	} else if (excl)
+
+		/*
+		 * The file does not exist. We could create it, but that
+		 * might result in a situation in which a file is created
+		 * with a mode that does not allow it to be opened (easily)
+		 * later. This is exactly the situation when called from
+		 * libc open(). The entry is first created with a mode, and
+		 * then opened with a set of flags. If the mode is rx, but
+		 * the flags say writeable, we run into a minor problem.
+		 * The easiest thing to do is just delay until until either
+		 a file_open is called, or the file is released.
+		 * Semantically incorrect, but this *is* supposed to be
+		 * a simple emulation. Even worse, all of the file operation
+		 * calls should check the flag too (ie: create, followed by
+		 * stat), and create the file. But, the only existing use of
+		 * oskit_dir_create is libc open, so lets not bother.
+		 */
+		fsf->create = 1;
+		fsf->mode   = mode;
+
+		/*
+		 * The directory holds one ref for the file object,
+		 * and we add another for the pointer going out.
+		 */
+		++fsf->count;
+		*out_file = &fsf->filei;
+		rc = 0;
+	}
+	else if (excl)
 		rc = OSKIT_EEXIST;
 	else {
 		oskit_file_addref(file);
 		*out_file = file;
 		rc = 0;
 	}
+   done:
 	return rc;
 }
 
@@ -725,8 +758,8 @@ fs_dir_link(oskit_dir_t *d, char *name, oskit_file_t *file)
 	 * Now try making the actual link.
 	 */
 	if (NATIVEOS(link)(fsf->pathname, newfsf->pathname) < 0) {
-		oskit_file_release(&newfsf->filei);		
-		return errno;
+		oskit_file_release(&newfsf->filei);
+		return native_to_oskit_error(native_errno);
 	}
 
 	/*
@@ -765,8 +798,8 @@ fs_dir_unlink(oskit_dir_t *d, const char *name)
 	strcpy(fname, fsd->pathname);
 	strcat(fname, name);
 	if (NATIVEOS(unlink)(fname) < 0)
-		return errno;
-	
+		return native_to_oskit_error(native_errno);
+
 	return 0;
 }
 
@@ -774,7 +807,52 @@ static OSKIT_COMDECL
 fs_dir_rename(oskit_dir_t *old_dir, char *old_name,
 	      oskit_dir_t *new_dir, char *new_name)
 {
-	return OSKIT_E_NOTIMPL;
+        struct fs_dir	*old_fsd = (struct fs_dir *) old_dir;
+        struct fs_dir	*new_fsd = (struct fs_dir *) new_dir;
+        struct fs_file	*new_fsf;
+        struct fs_file	*old_fsf;
+	char		old_fname[MAXPATHLEN];
+	char		new_fname[MAXPATHLEN];
+
+	old_fsf = (struct fs_file *) fs_lookup(old_fsd, old_name);
+	if (!old_fsf) return OSKIT_ENOENT;
+
+	new_fsf = (struct fs_file *) fs_lookup(new_fsd, new_name);
+	/*
+	 * Must be the same type if it already exists. Release our handle
+	 * on it before doing the actual disk operation.
+	 */
+	if (new_fsf) {
+		if (new_fsf->filei.ops != old_fsf->filei.ops) {
+			if (old_fsf->filei.ops == (void *)&fs_dir_ops)
+				return OSKIT_ENOTDIR;
+			else
+				return OSKIT_EISDIR;
+		}
+
+		fs_contents_unlink(new_fsd, (struct fs_dir *) new_fsf);
+		fs_file_release((oskit_file_t *) new_fsf);
+	}
+
+	/*
+	 * Now do the disk rename and let it return error.
+	 * This is slightly inconsistent, but okay.
+	 */
+	strcpy(old_fname, old_fsd->pathname);
+	strcat(old_fname, old_name);
+	strcpy(new_fname, new_fsd->pathname);
+	strcat(new_fname, new_name);
+	if (NATIVEOS(rename)(old_fname, new_fname) < 0)
+		return native_to_oskit_error(native_errno);
+
+	/*
+	 * Operation success. Release the source file now that disk has
+	 * really been changed.
+	 */
+	fs_contents_unlink(old_fsd, (struct fs_dir *) old_fsf);
+	fs_file_release((oskit_file_t *) old_fsf);
+
+	return 0;
 }
 
 static OSKIT_COMDECL
@@ -797,12 +875,12 @@ fs_dir_mkdir(oskit_dir_t *d, const char *name, oskit_mode_t mode)
 	strcpy(fname, fsd->pathname);
 	strcat(fname, name);
 	if (NATIVEOS(mkdir)(fname, mode) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	/*
 	 * Yep, so now make the fs_dir structure and link it in.
 	 */
-	if ((newfsd = fs_dir_allocate(fsd->pathname, name)) == NULL)
+	if ((newfsd = fs_dir_allocate(fsd->pathname, name, d)) == NULL)
 		return OSKIT_ENOMEM;
 
 	fs_contents_link(fsd, newfsd);
@@ -843,7 +921,7 @@ fs_dir_rmdir(oskit_dir_t *d, const char *name)
 	strcpy(fname, fsd->pathname);
 	strcat(fname, name);
 	if (NATIVEOS(rmdir)(fname) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	return 0;
 }
@@ -869,13 +947,16 @@ fs_dir_rmdir(oskit_dir_t *d, const char *name)
 static OSKIT_COMDECL
 fs_dir_getdirentries(oskit_dir_t *d, oskit_u32_t *inout_ofs,
 		     oskit_u32_t nentries,
-		     struct oskit_dirents *out_dirents)
+		     struct oskit_dirents **out_dirents)
 {
         struct fs_dir		*fsd = (struct fs_dir *) d;
 	char			buf[0x1000];
-	int			count, dcnt, doff, i;
+	int			count, doff, dcnt, i;
 	struct oskit_dirent	*dirents;
-	struct bsddirent	*dent;
+	oskit_size_t		dirents_size;
+	struct dirent		*dent;
+	oskit_error_t		rc;
+	off_t			offset;
 
 	/*
 	 * This is implemented using the underlying filesystem only.
@@ -885,28 +966,27 @@ fs_dir_getdirentries(oskit_dir_t *d, oskit_u32_t *inout_ofs,
 	 */
 	if (fsd->fd < 0) {
 		if ((fsd->fd = NATIVEOS(open)(fsd->pathname, O_RDONLY, 0)) < 0)
-			return errno;
-	}
-	else if (*inout_ofs == 0) {
-		if (NATIVEOS(lseek)(fsd->fd, 0, SEEK_SET) < 0)
-			return errno;
-	}
-	else if (*inout_ofs == -1) {
-		assert(fsd->fd >= 0);
-		NATIVEOS(close)(fsd->fd);
-		fsd->fd = -1;
-		out_dirents->dirents = NULL;
-		out_dirents->count = 0;
-		return 0;
+			return native_to_oskit_error(native_errno);
 	}
 
+	/*
+	 * basep is not read by getdirents.  At the end of the call,
+	 * it is set to the file position of the first entry.  Always
+	 * reset file pos, in case the file was closed or seeked.
+	 *
+	 * This behavior is documented in the freebsd manpage.  The
+	 * linux manpage says something else, but acts like BSD.
+	 */
+	NATIVEOS(lseek)(fsd->fd, *inout_ofs, SEEK_SET);
+
 	/*
 	 * Ask for a fixed size block of entries, and then convert that
 	 * into Oskit dirent structures.
 	 */
+	offset = *inout_ofs;
 	if ((count =
-	     NATIVEOS(getdirentries)(fsd->fd, buf, 0x1000, inout_ofs)) < 0)
-		return errno;
+	     NATIVEOS(getdirentries)(fsd->fd, buf, sizeof buf, &offset)) < 0)
+		return native_to_oskit_error(native_errno);
 
 	/*
 	 * No entries at all, so close it down.
@@ -914,53 +994,55 @@ fs_dir_getdirentries(oskit_dir_t *d, oskit_u32_t *inout_ofs,
 	if (count == 0) {
 		NATIVEOS(close)(fsd->fd);
 		fsd->fd = -1;
-		out_dirents->dirents = NULL;
-		out_dirents->count = 0;
+		*out_dirents = NULL;
 		return 0;
 	}
 
-	/*
-	 * There were entries, but inout_ofs comes back as zero, which
-	 * means there are no more to read. Need to indicate that for
-	 * the next call, which is going to try and read more.
-	 */
-	if (*inout_ofs == 0)
-		*inout_ofs = -1;
+	offset = NATIVEOS(lseek)(fsd->fd, 0, SEEK_CUR);
+	if (offset < 0) return native_to_oskit_error(native_errno);
+	*inout_ofs = offset;
 
 	/*
 	 * Count them up.
 	 */
-	dent = (struct bsddirent *) buf;
-	dcnt = 0;
+	dent = (struct dirent *) buf;
+	dirents_size = 0;
 	doff = 0;
+	dcnt = 0;
 	while (doff < count) {
+		/* XXX kludge that is always too big, but well-aligned */
+		dirents_size += offsetof(oskit_dirent_t, name[dent->d_reclen]);
 		doff += dent->d_reclen;
-		dent  = (struct bsddirent *) &buf[doff];
-		dcnt++;
+		dent  = (struct dirent *) &buf[doff];
+		++dcnt;
 	}
 
-	dirents = malloc(sizeof dirents[0] * dcnt);
+	dirents = malloc(dirents_size);
 	if (!dirents) {
-		out_dirents->dirents = NULL;
-		out_dirents->count = 0;
+		*out_dirents = NULL;
 		return OSKIT_ENOMEM;
 	}
-	out_dirents->dirents = dirents;
-	out_dirents->count = dcnt;
 
-	dent = (struct bsddirent *) buf;
-	for (i = 0, doff = 0; i < dcnt; i++, dirents++) {
-		/* copy filename in */
-		dirents->name = malloc(dent->d_namlen + 1);
-		if (dirents->name) {
-			memcpy(dirents->name, dent->d_name, dent->d_namlen);
-			dirents->name[dent->d_namlen] = 0;
-		}
-		dirents->ino = dent->d_fileno;
-
-		doff += dent->d_reclen;
-		dent  = (struct bsddirent *) &buf[doff];
+	/*
+	 * Create the dirents COM object first to make sure we can.
+	 */
+	if ((rc = fs_dirents_create(dirents, dcnt, out_dirents))) {
+		free(dirents);
+		*out_dirents = NULL;
+		return rc;
 	}
+
+	dent = (struct dirent *) buf;
+	for (i = 0, doff = 0; i < dcnt; i++) {
+		/* copy filename in */
+		dirents->namelen = DIRENTNAMLEN(dent);
+		dirents->ino = DIRENTINO(dent);
+		memcpy(dirents->name, dent->d_name, DIRENTNAMLEN(dent) + 1);
+		dirents = (void *)(&dirents->name[dent->d_reclen]);
+		doff += dent->d_reclen;
+		dent  = (struct dirent *) &buf[doff];
+	}
+
 	return 0;
 }
 
@@ -984,13 +1066,13 @@ fs_dir_symlink(oskit_dir_t *d, char *link_name, char *dest_name)
 
 	/*
 	 * Lets just create the symlink and not worry about caching
-	 * a handle on it. 
+	 * a handle on it.
 	 */
 	strcpy(fname, fsd->pathname);
 	strcat(fname, link_name);
 
 	if (NATIVEOS(symlink(dest_name, fname)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	return 0;
 }
@@ -1070,7 +1152,7 @@ fs_ofile_query(oskit_openfile_t *f,
 	}
 
 	*out_ihandle = NULL;
-	return OSKIT_E_NOINTERFACE;    
+	return OSKIT_E_NOINTERFACE;
 }
 
 static OSKIT_COMDECL_U
@@ -1090,7 +1172,7 @@ fs_ofile_release(oskit_openfile_t *f)
 {
 	struct fs_open	*ofile = (struct fs_open *) f;
 	unsigned	newcount;
-	
+
 	assert(ofile);
 	assert(ofile->count != 0);
 
@@ -1116,7 +1198,7 @@ fs_ofile_read(oskit_openfile_t *f,
 	assert(ofile->count != 0);
 
 	if ((count = NATIVEOS(read)(ofile->fd, buf, len)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	*out_actual = count;
 	return 0;
@@ -1133,7 +1215,7 @@ fs_ofile_write(oskit_openfile_t *f,
 	assert(ofile->count != 0);
 
 	if ((count = NATIVEOS(write)(ofile->fd, buf, len)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	*out_actual = count;
 	return 0;
@@ -1150,7 +1232,7 @@ fs_ofile_seek(oskit_openfile_t *f,
 	assert(ofile->count != 0);
 
 	if ((newpos = NATIVEOS(lseek)(ofile->fd, ofs, whence)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	*out_newpos = newpos;
 	return 0;
@@ -1166,10 +1248,10 @@ fs_ofile_setsize(oskit_openfile_t *f, oskit_u64_t new_size)
 	assert(ofile->count != 0);
 
 	if ((oldpos = NATIVEOS(lseek)(ofile->fd, new_size, SEEK_SET)) < 0)
-		return errno;
-	
+		return native_to_oskit_error(native_errno);
+
 	if (NATIVEOS(lseek)(ofile->fd, oldpos, SEEK_SET) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	return 0;
 }
@@ -1272,18 +1354,19 @@ oskit_unix_fs_init(char *rootname)
 		panic("oskit_unix_fs_init: Cannot access root `%s'", rootname);
 
         if (newfsd == NULL)
-		panic("oskit_unix_fs_init: Out of memory");		
-	
+		panic("oskit_unix_fs_init: Out of memory");
+
         memset(newfsd, 0, sizeof(*newfsd));
 
 	pathlen = strlen(rootname);
-	
+
 	if ((newfsd->pathname = malloc(pathlen + 2)) == NULL) {
 		free(newfsd);
 		return 0;
 	}
 	strcpy(newfsd->pathname, rootname);
-	strcat(newfsd->pathname, "/");
+	if (newfsd->pathname[strlen(newfsd->pathname) - 1] != '/')
+	    strcat(newfsd->pathname, "/");
 
 	newfsd->component  = NULL;
 	newfsd->complen    = 0;
@@ -1326,3 +1409,143 @@ fs_dumptree()
 {
 	fs_print_dir(fs_dir_rootdir);
 }
+
+/*
+ * oskit_dirents_t COM object implementation.
+ */
+struct fs_dirents {
+	oskit_dirents_t		direntsi;	/* COM dirents interface */
+	int			count;		/* Reference count */
+	oskit_dirent_t       *entries;	/* Array of entries */
+	int			dircount;	/* Number of entries */
+	int			index;		/* Current index pointer */
+};
+
+/*
+ * Methods.
+ */
+static struct oskit_dirents_ops fs_dirents_ops;
+
+static OSKIT_COMDECL
+fs_dirents_query(oskit_dirents_t *d,
+		 const oskit_iid_t *iid, void **out_ihandle)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	assert(fsdirents->count);
+
+        if (memcmp(iid, &oskit_iunknown_iid, sizeof(*iid)) == 0 ||
+            memcmp(iid, &oskit_dirents_iid, sizeof(*iid)) == 0) {
+                *out_ihandle = fsdirents;
+                ++fsdirents->count;
+                return 0;
+        }
+
+        *out_ihandle = NULL;
+        return OSKIT_E_NOINTERFACE;
+}
+
+static OSKIT_COMDECL_U
+fs_dirents_addref(oskit_dirents_t *d)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	assert(fsdirents->count);
+
+	return ++fsdirents->count;
+}
+
+static OSKIT_COMDECL_U
+fs_dirents_release(oskit_dirents_t *d)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	int i;
+
+	assert(fsdirents->count);
+
+	if (--fsdirents->count)
+		return fsdirents->count;
+
+	for (i = 0; i < fsdirents->dircount; i++)
+		free(fsdirents->entries[i].name);
+
+	free(fsdirents->entries);
+	sfree(fsdirents, sizeof(*fsdirents));
+	return 0;
+}
+
+static OSKIT_COMDECL
+fs_dirents_getcount(oskit_dirents_t *d, int *out_count)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	assert(fsdirents->count);
+
+	*out_count = fsdirents->dircount;
+
+	return 0;
+}
+
+static OSKIT_COMDECL
+fs_dirents_getnext(oskit_dirents_t *d, oskit_dirent_t *dirent)
+{
+	struct fs_dirents	*fsdirents = (struct fs_dirents *) d;
+	struct oskit_dirent  *idirent;
+
+	assert(fsdirents->count);
+
+	if (fsdirents->index >= fsdirents->dircount)
+		return OSKIT_EWOULDBLOCK;
+
+	idirent = &fsdirents->entries[fsdirents->index];
+
+	if (dirent->namelen < idirent->namelen + 1)
+		return OSKIT_ENAMETOOLONG;
+
+	dirent->ino     = idirent->ino;
+	dirent->namelen = idirent->namelen;
+	strcpy(dirent->name, idirent->name);
+
+	fsdirents->index++;
+	return 0;
+}
+
+static OSKIT_COMDECL
+fs_dirents_rewind(oskit_dirents_t *d)
+{
+	struct fs_dirents *fsdirents = (struct fs_dirents *) d;
+	assert(fsdirents->count);
+
+	fsdirents->index = 0;
+	return 0;
+}
+
+static struct oskit_dirents_ops fs_dirents_ops = {
+	fs_dirents_query,
+	fs_dirents_addref,
+	fs_dirents_release,
+	fs_dirents_getcount,
+	fs_dirents_getnext,
+	fs_dirents_rewind,
+};
+
+/*
+ * Create a oskit_dirents_t COM object from an array of oskit_dirent_t
+ * entries.
+ */
+static oskit_error_t
+fs_dirents_create(oskit_dirent_t *entries,
+		  int count, oskit_dirents_t **out_dirents)
+{
+        struct fs_dirents *fsdirents = smalloc(sizeof(*fsdirents));
+
+        if (fsdirents == NULL)
+                return OSKIT_ENOMEM;
+
+        fsdirents->count        = 1;
+        fsdirents->dircount     = count;
+        fsdirents->entries      = entries;
+        fsdirents->index        = 0;
+	fsdirents->direntsi.ops = &fs_dirents_ops;
+
+	*out_dirents = &fsdirents->direntsi;
+
+	return 0;
+}
