@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 University of Utah and the Flux Group.
+ * Copyright (c) 1998, 1999, 2000 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -24,7 +24,7 @@
  * Use
  *
  *   oskit_error_t
- *   oskit_freebsd_native_net_init(oskit_socket_factory_t **f);
+ *   oskit_native_net_init(oskit_socket_factory_t **f);
  *
  * to access it.
  */
@@ -36,6 +36,7 @@
 #include <oskit/io/asyncio.h>
 #include <oskit/io/bufio_stream.h>
 #include <oskit/com/listener_mgr.h>
+#include <oskit/clientos.h>
 #include <oskit/net/socket.h>
 
 #include "native.h"
@@ -49,6 +50,7 @@
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <string.h>
+
 #include <errno.h>
 
 /*
@@ -239,6 +241,22 @@ socket_pathconf(oskit_socket_t *f, oskit_s32_t option, oskit_s32_t *out_val)
         return OSKIT_ENOTSUP;
 }
 
+#ifdef NATIVE_HAVE_SA_LEN
+#define SA_FIX_OUT(sa, salen)	((void)0)
+#define	SA_IN(sa, salen)	struct sockaddr *const SA = (void *)(sa);
+#else
+#define SA_FIX_OUT(sa, len) \
+	((((struct oskit_sockaddr *) (sa))->sa_family = \
+	  ((struct sockaddr *) (sa))->sa_family), \
+	 ((struct oskit_sockaddr *) (sa))->sa_len = (len))
+#define SA_IN(sa, salen)						      \
+	struct sockaddr _sa, *const SA = &_sa;				      \
+	assert((salen) <= sizeof _sa);					      \
+	_sa.sa_family = (sa)->sa_family;				      \
+	memcpy(_sa.sa_data, (sa)->sa_data,				      \
+	       (salen) - offsetof(struct oskit_sockaddr, sa_data))
+#endif
+
 /*** Operations specific to oskit_socket_t ***/
 static OSKIT_COMDECL
 socket_accept(oskit_socket_t *s, struct oskit_sockaddr *name,
@@ -247,9 +265,14 @@ socket_accept(oskit_socket_t *s, struct oskit_sockaddr *name,
         oskit_sockimpl_t *b;
 	int	nfd;
 
+	assert(offsetof(struct oskit_sockaddr, sa_data) ==
+	       offsetof(struct sockaddr, sa_data));
+
 	if ((nfd = threaded_accept(((oskit_sockimpl_t*)s)->fd,
 				   (struct sockaddr *)name, anamelen)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
+
+	SA_FIX_OUT(name, *anamelen);
 
         /* create a new sockimpl */
         b = create_sockimpl(nfd);               /* XXX error check */
@@ -262,8 +285,12 @@ static OSKIT_COMDECL
 socket_bind(oskit_socket_t *s, const struct oskit_sockaddr *name,
         oskit_size_t namelen)
 {
-        return NATIVEOS(bind)(((oskit_sockimpl_t *)s)->fd,
-                                   (struct sockaddr *)name, namelen);
+	SA_IN(name, namelen);
+        if (NATIVEOS(bind)(((oskit_sockimpl_t *)s)->fd,
+			   SA, namelen) < 0)
+		return native_to_oskit_error(native_errno);
+
+	return 0;
 }
 
 static OSKIT_COMDECL
@@ -271,9 +298,10 @@ socket_connect(oskit_socket_t *s, const struct oskit_sockaddr *name,
         oskit_size_t namelen)
 {
         oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
+	SA_IN(name, namelen);
 
-	if (threaded_connect(b->fd, (struct sockaddr *)name, namelen) < 0)
-		return errno;
+	if (threaded_connect(b->fd, SA, namelen) < 0)
+		return native_to_oskit_error(native_errno);
 
         return 0;
 }
@@ -281,25 +309,34 @@ socket_connect(oskit_socket_t *s, const struct oskit_sockaddr *name,
 static OSKIT_COMDECL
 socket_listen(oskit_socket_t *s, oskit_u32_t n)
 {
-        int    rc;
-        rc = NATIVEOS(listen)(((oskit_sockimpl_t *)s)->fd, n);
-        return (rc);
+	if (NATIVEOS(listen)(((oskit_sockimpl_t *)s)->fd, n) < 0)
+	    	return native_to_oskit_error(native_errno);
+
+	return 0;
 }
 
 static OSKIT_COMDECL
 socket_getsockname(oskit_socket_t *s, struct oskit_sockaddr *asa,
         oskit_size_t *alen)
 {
-        return NATIVEOS(getsockname)(((oskit_sockimpl_t *)s)->fd,
-                                     (struct sockaddr *)asa, alen);
+    	if (NATIVEOS(getsockname)(((oskit_sockimpl_t *)s)->fd,
+				  (struct sockaddr *)asa, alen) < 0)
+	    	return native_to_oskit_error(native_errno);
+	SA_FIX_OUT(asa, *alen);
+	return 0;
 }
 
+/* XXX translate sockopts! */
 static OSKIT_COMDECL
 socket_setsockopt(oskit_socket_t *s, oskit_u32_t level, oskit_u32_t name,
                 const void *val, oskit_size_t valsize)
 {
-        return NATIVEOS(setsockopt)(((oskit_sockimpl_t *)s)->fd,
-                                         level, name, val, valsize);
+    	if (NATIVEOS(setsockopt)(((oskit_sockimpl_t *)s)->fd,
+				     level, name, val, valsize) < 0)
+	    	return native_to_oskit_error(native_errno);
+
+	return 0;
+
 }
 
 static OSKIT_COMDECL
@@ -307,13 +344,13 @@ socket_sendto(oskit_socket_t *s, const void *msg, oskit_size_t len,
                 oskit_u32_t flags, const struct oskit_sockaddr *to,
                 oskit_size_t tolen, oskit_size_t *retval)
 {
-        oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
 	int		  count;
+	SA_IN(to, tolen);
 
 	if ((count = threaded_sendto(((oskit_sockimpl_t *)s)->fd,
                                      msg, len, flags,
-                                     (struct sockaddr *)to, tolen)) < 0)
-		return errno;
+                                     SA, tolen)) < 0)
+		return native_to_oskit_error(native_errno);
 
 	*retval = (oskit_size_t) count;
 	return 0;
@@ -325,14 +362,13 @@ socket_recvfrom(oskit_socket_t *s, void *buf,
                 struct oskit_sockaddr *from, oskit_size_t *fromlen,
                 oskit_size_t *retval)
 {
-        oskit_sockimpl_t *b = (oskit_sockimpl_t *)s;
 	int		  count;
 
 	if ((count = threaded_recvfrom(((oskit_sockimpl_t *)s)->fd,
                                        buf, len, flags,
                                        (struct sockaddr *)from, fromlen)) < 0)
-		return errno;
-
+		return native_to_oskit_error(native_errno);
+	SA_FIX_OUT(from, *fromlen);
 	*retval = (oskit_size_t) count;
 	return 0;
 }
@@ -363,16 +399,19 @@ socket_getpeername(oskit_socket_t *f,
         struct oskit_sockaddr *asa, oskit_size_t *alen)
 {
         oskit_sockimpl_t *s = (oskit_sockimpl_t *)f;
-        return NATIVEOS(getpeername)(s->fd, (struct sockaddr *)asa, alen);
+        if (NATIVEOS(getpeername)(s->fd, (struct sockaddr *)asa, alen) < 0)
+	    	return native_to_oskit_error(native_errno);
+	SA_FIX_OUT(asa, *alen);
+	return 0;
 }
 
 static OSKIT_COMDECL
 socket_shutdown(oskit_socket_t *f, oskit_u32_t how)
 {
-        int    rc;
+        if (NATIVEOS(shutdown)(((oskit_sockimpl_t *)f)->fd, how) < 0)
+	    	return native_to_oskit_error(native_errno);
 
-        rc = NATIVEOS(shutdown)(((oskit_sockimpl_t *)f)->fd, how);
-        return (rc);
+	return 0;
 }
 
 /*******************************************************/
@@ -388,7 +427,7 @@ opensocket_read(oskit_stream_t *f, void *buf, oskit_u32_t len,
 	int		  count;
 
 	if ((count = threaded_read(s->fd, buf, len)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	*out_actual = count;
 	return 0;
@@ -402,7 +441,7 @@ opensocket_write(oskit_stream_t *f, const void *buf,
 	int		  count;
 
 	if ((count = threaded_write(s->fd, buf, len)) < 0)
-		return errno;
+		return native_to_oskit_error(native_errno);
 
 	*out_actual = count;
 	return 0;
@@ -650,7 +689,8 @@ asyncio_readable(oskit_asyncio_t *f)
 {
         oskit_sockimpl_t *si = (oskit_sockimpl_t *)(f-2);
 	int b;
-	int rc = NATIVEOS(ioctl)(si->fd, FIONREAD, &b);
+	int rc;
+	rc = NATIVEOS(ioctl)(si->fd, FIONREAD, &b);
 	return b;	/* XXX ??? */
 }
 
@@ -774,13 +814,12 @@ socket_factory_create(
         oskit_u32_t domain,
         oskit_u32_t type, oskit_u32_t protocol, oskit_socket_t **aso)
 {
-        oskit_error_t   rc;
         oskit_sockimpl_t *b;
 	int fd;
 
         fd = NATIVEOS(socket)(domain, type, protocol);
         if (fd < 0)
-                return (errno);
+                return native_to_oskit_error(native_errno);
 
         b = create_sockimpl(fd);
         *aso = &b->ioi;
@@ -797,12 +836,11 @@ socket_factory_create_pair(
         oskit_u32_t type, oskit_u32_t protocol,
 	oskit_socket_t **aso1, oskit_socket_t **aso2)
 {
-        oskit_error_t   rc;
         oskit_sockimpl_t *b;
 	int fds[2];
 
-        if (NATIVEOS(socket)(domain, type, protocol, fds) < 0);
-                return (errno);
+        if (NATIVEOS(socketpair)(domain, type, protocol, fds) < 0);
+                return native_to_oskit_error(native_errno);
 
         b = create_sockimpl(fds[0]);
         *aso1 = &b->ioi;
@@ -820,14 +858,15 @@ static struct oskit_socket_factory_ops sf_ops = {
 };
 
 static
-struct oskit_socket_factory oskit_freebsd_native_socket_factory = { &sf_ops };
+struct oskit_socket_factory oskit_native_socket_factory = { &sf_ops };
 
 oskit_error_t
-oskit_freebsd_native_net_init(oskit_socket_factory_t **f)
+oskit_native_net_init(oskit_socket_factory_t **f)
 {
-	extern char hostname[256];
+	char hostname[256];
 
 	NATIVEOS(gethostname)(hostname, 256);
-        *f = &oskit_freebsd_native_socket_factory;
+	oskit_clientos_sethostname(hostname, 256);
+        *f = &oskit_native_socket_factory;
         return 0;
 }
