@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1998,2000 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -14,17 +14,15 @@
  * received a copy of the GPL along with the OSKit; see the file COPYING.  If
  * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
  */
-/* 
+/*
  * "interrupt" handler
  */
 struct sigcontext;
 
-int     
+int
 set_signal_handler(int sig, void (*handler)(int, int, struct sigcontext *));
 
-int add_ethernet_device(char *name);
-
-int open_eth(char *ifname, char *myaddr, 
+int open_eth(char *ifname, char *myaddr,
 	unsigned int *buflen, int no_ieee802_3);
 
 void register_async_fd(int fd, unsigned iotype,
@@ -33,3 +31,12 @@ void unregister_async_fd(int fd);
 
 #define IOTYPE_READ	1
 #define IOTYPE_WRITE	2
+
+int threaded_read(int fd, void* buf, size_t len);
+int threaded_write(int fd, const void* buf, size_t len);
+int threaded_connect(int fd, struct sockaddr* addr, size_t len);
+int threaded_accept(int fd, struct sockaddr* addr, size_t* len);
+int threaded_recvfrom(int fd, void* buf, size_t len, int flags,
+		      struct sockaddr* from, int* fromlen);
+int threaded_sendto(int fd, const void* buf, size_t len, int flags,
+		    struct sockaddr* to, int tolen);
