@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/types.h>
 #include <sys/mman.h>
 #include <signal.h>
 #include "vga.h"
@@ -37,6 +38,9 @@
 #include "interface.h"
 #include "accel.h"
 
+#ifdef OSKIT
+#include "osenv.h"
+#endif
 
 /* Extended registers. */
 
