@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997-1998 University of Utah and the Flux Group.
+ * Copyright (c) 1997-1999 University of Utah and the Flux Group.
  * All rights reserved.
  * 
  * This file is part of the Flux OSKit.  The OSKit is free software, also known
@@ -22,7 +22,7 @@
 #include "xf86_OSlib.h"
 #include <string.h>
 
-#include <oskit/dev/dev.h>
+#include "osenv.h"
 
 
 int
@@ -36,7 +36,8 @@ xf86ReadBIOS(Base, Offset, Buf, Len)
 	int rc;
 
 	
-	if (rc = osenv_mem_map_phys(Base + Offset, Len, &base, 0))
+	if (rc = oskit_osenv_mem_mapphys(oskit_x11_oskit_osenv_mem,
+					 Base + Offset, Len, &base, 0))
 		return -1;
 
 	bcopy(base, Buf, Len);
